from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import thread_stream_when

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  wire [32-1:0] myaxi_awaddr;
  wire [8-1:0] myaxi_awlen;
  wire myaxi_awvalid;
  reg myaxi_awready;
  wire [32-1:0] myaxi_wdata;
  wire [4-1:0] myaxi_wstrb;
  wire myaxi_wlast;
  wire myaxi_wvalid;
  reg myaxi_wready;
  wire [32-1:0] myaxi_araddr;
  wire [8-1:0] myaxi_arlen;
  wire myaxi_arvalid;
  reg myaxi_arready;
  reg [32-1:0] myaxi_rdata;
  reg myaxi_rlast;
  reg myaxi_rvalid;
  wire myaxi_rready;
  wire [32-1:0] memory_awaddr;
  wire [8-1:0] memory_awlen;
  wire memory_awvalid;
  reg memory_awready;
  wire [32-1:0] memory_wdata;
  wire [4-1:0] memory_wstrb;
  wire memory_wlast;
  wire memory_wvalid;
  reg memory_wready;
  wire [32-1:0] memory_araddr;
  wire [8-1:0] memory_arlen;
  wire memory_arvalid;
  reg memory_arready;
  reg [32-1:0] memory_rdata;
  reg memory_rlast;
  reg memory_rvalid;
  wire memory_rready;
  reg [8-1:0] _memory_mem [0:2**20-1];

  initial begin
    $readmemh("_memory_memimg_.out", _memory_mem);
  end

  reg [32-1:0] _memory_fsm;
  localparam _memory_fsm_init = 0;
  reg [33-1:0] _write_count;
  reg [32-1:0] _write_addr;
  reg [33-1:0] _read_count;
  reg [32-1:0] _read_addr;
  reg [33-1:0] _sleep_count;
  reg [32-1:0] _d1__memory_fsm;
  reg __memory_fsm_cond_100_0_1;
  reg __memory_fsm_cond_200_1_1;
  reg __memory_fsm_cond_211_2_1;
  assign memory_awaddr = myaxi_awaddr;
  assign memory_awlen = myaxi_awlen;
  assign memory_awvalid = myaxi_awvalid;

  always @(*) begin
    myaxi_awready <= memory_awready;
  end

  assign memory_wdata = myaxi_wdata;
  assign memory_wstrb = myaxi_wstrb;
  assign memory_wlast = myaxi_wlast;
  assign memory_wvalid = myaxi_wvalid;

  always @(*) begin
    myaxi_wready <= memory_wready;
  end

  assign memory_araddr = myaxi_araddr;
  assign memory_arlen = myaxi_arlen;
  assign memory_arvalid = myaxi_arvalid;

  always @(*) begin
    myaxi_arready <= memory_arready;
  end


  always @(*) begin
    myaxi_rdata <= memory_rdata;
  end


  always @(*) begin
    myaxi_rlast <= memory_rlast;
  end


  always @(*) begin
    myaxi_rvalid <= memory_rvalid;
  end

  assign memory_rready = myaxi_rready;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .myaxi_awaddr(myaxi_awaddr),
    .myaxi_awlen(myaxi_awlen),
    .myaxi_awvalid(myaxi_awvalid),
    .myaxi_awready(myaxi_awready),
    .myaxi_wdata(myaxi_wdata),
    .myaxi_wstrb(myaxi_wstrb),
    .myaxi_wlast(myaxi_wlast),
    .myaxi_wvalid(myaxi_wvalid),
    .myaxi_wready(myaxi_wready),
    .myaxi_araddr(myaxi_araddr),
    .myaxi_arlen(myaxi_arlen),
    .myaxi_arvalid(myaxi_arvalid),
    .myaxi_arready(myaxi_arready),
    .myaxi_rdata(myaxi_rdata),
    .myaxi_rlast(myaxi_rlast),
    .myaxi_rvalid(myaxi_rvalid),
    .myaxi_rready(myaxi_rready)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    memory_awready = 0;
    memory_wready = 0;
    memory_arready = 0;
    memory_rdata = 0;
    memory_rlast = 0;
    memory_rvalid = 0;
    _memory_fsm = _memory_fsm_init;
    _write_count = 0;
    _write_addr = 0;
    _read_count = 0;
    _read_addr = 0;
    _sleep_count = 0;
    _d1__memory_fsm = _memory_fsm_init;
    __memory_fsm_cond_100_0_1 = 0;
    __memory_fsm_cond_200_1_1 = 0;
    __memory_fsm_cond_211_2_1 = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #100000;
    $finish;
  end

  localparam _memory_fsm_200 = 200;
  localparam _memory_fsm_201 = 201;
  localparam _memory_fsm_202 = 202;
  localparam _memory_fsm_203 = 203;
  localparam _memory_fsm_204 = 204;
  localparam _memory_fsm_205 = 205;
  localparam _memory_fsm_206 = 206;
  localparam _memory_fsm_207 = 207;
  localparam _memory_fsm_208 = 208;
  localparam _memory_fsm_209 = 209;
  localparam _memory_fsm_210 = 210;
  localparam _memory_fsm_211 = 211;
  localparam _memory_fsm_100 = 100;
  localparam _memory_fsm_101 = 101;
  localparam _memory_fsm_102 = 102;
  localparam _memory_fsm_103 = 103;
  localparam _memory_fsm_104 = 104;
  localparam _memory_fsm_105 = 105;
  localparam _memory_fsm_106 = 106;
  localparam _memory_fsm_107 = 107;
  localparam _memory_fsm_108 = 108;
  localparam _memory_fsm_109 = 109;
  localparam _memory_fsm_110 = 110;
  localparam _memory_fsm_111 = 111;
  localparam _memory_fsm_112 = 112;

  always @(posedge CLK) begin
    if(RST) begin
      _memory_fsm <= _memory_fsm_init;
      _d1__memory_fsm <= _memory_fsm_init;
      memory_awready <= 0;
      _write_addr <= 0;
      _write_count <= 0;
      __memory_fsm_cond_100_0_1 <= 0;
      memory_wready <= 0;
      memory_arready <= 0;
      _read_addr <= 0;
      _read_count <= 0;
      __memory_fsm_cond_200_1_1 <= 0;
      memory_rdata[7:0] <= (0 >> 0) & { 8{ 1'd1 } };
      memory_rdata[15:8] <= (0 >> 8) & { 8{ 1'd1 } };
      memory_rdata[23:16] <= (0 >> 16) & { 8{ 1'd1 } };
      memory_rdata[31:24] <= (0 >> 24) & { 8{ 1'd1 } };
      memory_rvalid <= 0;
      memory_rlast <= 0;
      __memory_fsm_cond_211_2_1 <= 0;
      memory_rdata <= 0;
      _sleep_count <= 0;
    end else begin
      _sleep_count <= _sleep_count + 1;
      if(_sleep_count == 3) begin
        _sleep_count <= 0;
      end 
      _d1__memory_fsm <= _memory_fsm;
      case(_d1__memory_fsm)
        _memory_fsm_100: begin
          if(__memory_fsm_cond_100_0_1) begin
            memory_awready <= 0;
          end 
        end
        _memory_fsm_200: begin
          if(__memory_fsm_cond_200_1_1) begin
            memory_arready <= 0;
          end 
        end
        _memory_fsm_211: begin
          if(__memory_fsm_cond_211_2_1) begin
            memory_rvalid <= 0;
            memory_rlast <= 0;
          end 
        end
      endcase
      case(_memory_fsm)
        _memory_fsm_init: begin
          if(memory_awvalid) begin
            _memory_fsm <= _memory_fsm_100;
          end 
          if(memory_arvalid) begin
            _memory_fsm <= _memory_fsm_200;
          end 
        end
        _memory_fsm_100: begin
          if(memory_awvalid) begin
            memory_awready <= 1;
            _write_addr <= memory_awaddr;
            _write_count <= memory_awlen + 1;
          end 
          __memory_fsm_cond_100_0_1 <= 1;
          if(!memory_awvalid) begin
            _memory_fsm <= _memory_fsm_init;
          end 
          if(memory_awvalid) begin
            _memory_fsm <= _memory_fsm_101;
          end 
        end
        _memory_fsm_101: begin
          _memory_fsm <= _memory_fsm_102;
        end
        _memory_fsm_102: begin
          _memory_fsm <= _memory_fsm_103;
        end
        _memory_fsm_103: begin
          _memory_fsm <= _memory_fsm_104;
        end
        _memory_fsm_104: begin
          _memory_fsm <= _memory_fsm_105;
        end
        _memory_fsm_105: begin
          _memory_fsm <= _memory_fsm_106;
        end
        _memory_fsm_106: begin
          _memory_fsm <= _memory_fsm_107;
        end
        _memory_fsm_107: begin
          _memory_fsm <= _memory_fsm_108;
        end
        _memory_fsm_108: begin
          _memory_fsm <= _memory_fsm_109;
        end
        _memory_fsm_109: begin
          _memory_fsm <= _memory_fsm_110;
        end
        _memory_fsm_110: begin
          _memory_fsm <= _memory_fsm_111;
        end
        _memory_fsm_111: begin
          memory_wready <= 1;
          _memory_fsm <= _memory_fsm_112;
        end
        _memory_fsm_112: begin
          if(memory_wvalid && memory_wstrb[0]) begin
            _memory_mem[_write_addr + 0] <= memory_wdata[7:0];
          end 
          if(memory_wvalid && memory_wstrb[1]) begin
            _memory_mem[_write_addr + 1] <= memory_wdata[15:8];
          end 
          if(memory_wvalid && memory_wstrb[2]) begin
            _memory_mem[_write_addr + 2] <= memory_wdata[23:16];
          end 
          if(memory_wvalid && memory_wstrb[3]) begin
            _memory_mem[_write_addr + 3] <= memory_wdata[31:24];
          end 
          if(memory_wvalid && memory_wready) begin
            _write_addr <= _write_addr + 4;
            _write_count <= _write_count - 1;
          end 
          if(_sleep_count == 3) begin
            memory_wready <= 0;
          end else begin
            memory_wready <= 1;
          end
          if(memory_wvalid && memory_wready && (_write_count == 1)) begin
            memory_wready <= 0;
          end 
          if(memory_wvalid && memory_wready && (_write_count == 1)) begin
            _memory_fsm <= _memory_fsm_init;
          end 
        end
        _memory_fsm_200: begin
          if(memory_arvalid) begin
            memory_arready <= 1;
            _read_addr <= memory_araddr;
            _read_count <= memory_arlen + 1;
          end 
          __memory_fsm_cond_200_1_1 <= 1;
          if(!memory_arvalid) begin
            _memory_fsm <= _memory_fsm_init;
          end 
          if(memory_arvalid) begin
            _memory_fsm <= _memory_fsm_201;
          end 
        end
        _memory_fsm_201: begin
          _memory_fsm <= _memory_fsm_202;
        end
        _memory_fsm_202: begin
          _memory_fsm <= _memory_fsm_203;
        end
        _memory_fsm_203: begin
          _memory_fsm <= _memory_fsm_204;
        end
        _memory_fsm_204: begin
          _memory_fsm <= _memory_fsm_205;
        end
        _memory_fsm_205: begin
          _memory_fsm <= _memory_fsm_206;
        end
        _memory_fsm_206: begin
          _memory_fsm <= _memory_fsm_207;
        end
        _memory_fsm_207: begin
          _memory_fsm <= _memory_fsm_208;
        end
        _memory_fsm_208: begin
          _memory_fsm <= _memory_fsm_209;
        end
        _memory_fsm_209: begin
          _memory_fsm <= _memory_fsm_210;
        end
        _memory_fsm_210: begin
          _memory_fsm <= _memory_fsm_211;
        end
        _memory_fsm_211: begin
          if(memory_rready | !memory_rvalid) begin
            memory_rdata[7:0] <= _memory_mem[_read_addr + 0];
          end 
          if(memory_rready | !memory_rvalid) begin
            memory_rdata[15:8] <= _memory_mem[_read_addr + 1];
          end 
          if(memory_rready | !memory_rvalid) begin
            memory_rdata[23:16] <= _memory_mem[_read_addr + 2];
          end 
          if(memory_rready | !memory_rvalid) begin
            memory_rdata[31:24] <= _memory_mem[_read_addr + 3];
          end 
          if((_sleep_count < 3) && (_read_count > 0) && memory_rready | !memory_rvalid) begin
            memory_rvalid <= 1;
            _read_addr <= _read_addr + 4;
            _read_count <= _read_count - 1;
          end 
          if((_sleep_count < 3) && (_read_count == 1) && memory_rready | !memory_rvalid) begin
            memory_rlast <= 1;
          end 
          __memory_fsm_cond_211_2_1 <= 1;
          if(memory_rvalid && !memory_rready) begin
            memory_rdata <= memory_rdata;
            memory_rlast <= memory_rlast;
          end 
          if(memory_rvalid && memory_rready && (_read_count == 0)) begin
            _memory_fsm <= _memory_fsm_init;
          end 
        end
      endcase
    end
  end


endmodule



module blinkled
(
  input CLK,
  input RST,
  output reg [32-1:0] myaxi_awaddr,
  output reg [8-1:0] myaxi_awlen,
  output reg myaxi_awvalid,
  input myaxi_awready,
  output reg [32-1:0] myaxi_wdata,
  output reg [4-1:0] myaxi_wstrb,
  output reg myaxi_wlast,
  output reg myaxi_wvalid,
  input myaxi_wready,
  output reg [32-1:0] myaxi_araddr,
  output reg [8-1:0] myaxi_arlen,
  output reg myaxi_arvalid,
  input myaxi_arready,
  input [32-1:0] myaxi_rdata,
  input myaxi_rlast,
  input myaxi_rvalid,
  output myaxi_rready
);

  reg [10-1:0] ram_a_0_addr;
  wire [32-1:0] ram_a_0_rdata;
  reg [32-1:0] ram_a_0_wdata;
  reg ram_a_0_wenable;

  ram_a
  inst_ram_a
  (
    .CLK(CLK),
    .ram_a_0_addr(ram_a_0_addr),
    .ram_a_0_rdata(ram_a_0_rdata),
    .ram_a_0_wdata(ram_a_0_wdata),
    .ram_a_0_wenable(ram_a_0_wenable)
  );

  reg [10-1:0] ram_b_0_addr;
  wire [32-1:0] ram_b_0_rdata;
  reg [32-1:0] ram_b_0_wdata;
  reg ram_b_0_wenable;

  ram_b
  inst_ram_b
  (
    .CLK(CLK),
    .ram_b_0_addr(ram_b_0_addr),
    .ram_b_0_rdata(ram_b_0_rdata),
    .ram_b_0_wdata(ram_b_0_wdata),
    .ram_b_0_wenable(ram_b_0_wenable)
  );

  reg [10-1:0] ram_c_0_addr;
  wire [32-1:0] ram_c_0_rdata;
  reg [32-1:0] ram_c_0_wdata;
  reg ram_c_0_wenable;

  ram_c
  inst_ram_c
  (
    .CLK(CLK),
    .ram_c_0_addr(ram_c_0_addr),
    .ram_c_0_rdata(ram_c_0_rdata),
    .ram_c_0_wdata(ram_c_0_wdata),
    .ram_c_0_wenable(ram_c_0_wenable)
  );

  reg [32-1:0] th_comp;
  localparam th_comp_init = 0;
  reg signed [32-1:0] _th_comp_size_0;
  reg signed [32-1:0] _th_comp_offset_1;
  reg [10-1:0] _tmp_0;
  reg [32-1:0] _tmp_1;
  reg [32-1:0] _tmp_2;
  reg [32-1:0] _tmp_3;
  reg [32-1:0] _tmp_fsm_0;
  localparam _tmp_fsm_0_init = 0;
  reg [9-1:0] _tmp_4;
  reg _myaxi_cond_0_1;
  wire _tmp_5;
  wire _tmp_6;
  assign _tmp_5 = 1 && _tmp_ready_9;
  assign _tmp_6 = 1;
  assign myaxi_rready = _tmp_5 && _tmp_6 || _tmp_15 && _tmp_16 || _tmp_72 && _tmp_73 || _tmp_82 && _tmp_83;
  reg [33-1:0] _tmp_7;
  reg _tmp_8;
  wire [32-1:0] _tmp_data_9;
  wire _tmp_valid_9;
  wire _tmp_ready_9;
  assign _tmp_ready_9 = (_tmp_7 > 0) && !_tmp_8;
  reg _ram_a_cond_0_1;
  reg [10-1:0] _tmp_10;
  reg [32-1:0] _tmp_11;
  reg [32-1:0] _tmp_12;
  reg [32-1:0] _tmp_13;
  reg [32-1:0] _tmp_fsm_1;
  localparam _tmp_fsm_1_init = 0;
  reg [9-1:0] _tmp_14;
  reg _myaxi_cond_1_1;
  wire _tmp_15;
  wire _tmp_16;
  assign _tmp_15 = 1 && _tmp_ready_19;
  assign _tmp_16 = 1;
  reg [33-1:0] _tmp_17;
  reg _tmp_18;
  wire [32-1:0] _tmp_data_19;
  wire _tmp_valid_19;
  wire _tmp_ready_19;
  assign _tmp_ready_19 = (_tmp_17 > 0) && !_tmp_18;
  reg _ram_b_cond_0_1;
  reg _mystream_flag_2;
  reg [32-1:0] _mystream_fsm_3;
  localparam _mystream_fsm_3_init = 0;
  reg _tmp_20;
  reg _tmp_21;
  wire _tmp_22;
  wire _tmp_23;
  assign _tmp_22 = 1 && ((_tmp_ready_115 || !_tmp_valid_115) && (_tmp_20 && _tmp_32));
  assign _tmp_23 = 1;
  localparam _tmp_24 = 1;
  wire [_tmp_24-1:0] _tmp_25;
  assign _tmp_25 = (_tmp_22 || !_tmp_20) && (_tmp_23 || !_tmp_21);
  reg [_tmp_24-1:0] __tmp_25_1;
  wire [32-1:0] _tmp_26;
  reg [32-1:0] __tmp_26_1;
  assign _tmp_26 = (__tmp_25_1)? ram_a_0_rdata : __tmp_26_1;
  reg [33-1:0] _tmp_27;
  reg _tmp_28;
  reg _tmp_29;
  reg _tmp_30;
  reg _tmp_31;
  reg _mystream_flag_4;
  reg [32-1:0] _mystream_fsm_5;
  localparam _mystream_fsm_5_init = 0;
  reg _tmp_32;
  reg _tmp_33;
  wire _tmp_34;
  wire _tmp_35;
  assign _tmp_34 = 1 && ((_tmp_ready_115 || !_tmp_valid_115) && (_tmp_20 && _tmp_32));
  assign _tmp_35 = 1;
  localparam _tmp_36 = 1;
  wire [_tmp_36-1:0] _tmp_37;
  assign _tmp_37 = (_tmp_34 || !_tmp_32) && (_tmp_35 || !_tmp_33);
  reg [_tmp_36-1:0] __tmp_37_1;
  wire [32-1:0] _tmp_38;
  reg [32-1:0] __tmp_38_1;
  assign _tmp_38 = (__tmp_37_1)? ram_b_0_rdata : __tmp_38_1;
  reg [33-1:0] _tmp_39;
  reg _tmp_40;
  reg _tmp_41;
  reg _tmp_42;
  reg _tmp_43;
  reg _mystream_flag_6;
  reg [32-1:0] _mystream_fsm_7;
  localparam _mystream_fsm_7_init = 0;
  reg [2-1:0] _tmp_44;
  reg _tmp_45;
  wire [32-1:0] _tmp_data_46;
  wire _tmp_valid_46;
  wire _tmp_ready_46;
  assign _tmp_ready_46 = (_tmp_44 > 0) && !_tmp_45;
  wire [1-1:0] _tmp_data_47;
  wire _tmp_valid_47;
  wire _tmp_ready_47;
  assign _tmp_ready_47 = (_tmp_44 > 0) && !_tmp_45;
  reg _ram_c_cond_0_1;
  reg [10-1:0] _tmp_48;
  reg [32-1:0] _tmp_49;
  reg [32-1:0] _tmp_50;
  reg [32-1:0] _tmp_51;
  reg [32-1:0] _tmp_fsm_2;
  localparam _tmp_fsm_2_init = 0;
  reg [9-1:0] _tmp_52;
  reg _myaxi_cond_2_1;
  reg _tmp_53;
  reg _tmp_54;
  wire _tmp_55;
  wire _tmp_56;
  assign _tmp_55 = 1 && _tmp_ready_66;
  assign _tmp_56 = 1;
  localparam _tmp_57 = 1;
  wire [_tmp_57-1:0] _tmp_58;
  assign _tmp_58 = (_tmp_55 || !_tmp_53) && (_tmp_56 || !_tmp_54);
  reg [_tmp_57-1:0] __tmp_58_1;
  wire [32-1:0] _tmp_59;
  reg [32-1:0] __tmp_59_1;
  assign _tmp_59 = (__tmp_58_1)? ram_c_0_rdata : __tmp_59_1;
  reg [33-1:0] _tmp_60;
  reg _tmp_61;
  reg _tmp_62;
  reg _tmp_63;
  reg _tmp_64;
  reg _tmp_65;
  wire [32-1:0] _tmp_data_66;
  wire _tmp_valid_66;
  wire _tmp_ready_66;
  assign _tmp_ready_66 = (_tmp_fsm_2 == 3) && ((_tmp_52 > 0) && (myaxi_wready || !myaxi_wvalid));
  reg _myaxi_cond_3_1;
  reg [10-1:0] _tmp_67;
  reg [32-1:0] _tmp_68;
  reg [32-1:0] _tmp_69;
  reg [32-1:0] _tmp_70;
  reg [32-1:0] _tmp_fsm_3;
  localparam _tmp_fsm_3_init = 0;
  reg [9-1:0] _tmp_71;
  reg _myaxi_cond_4_1;
  wire _tmp_72;
  wire _tmp_73;
  assign _tmp_72 = 1 && _tmp_ready_76;
  assign _tmp_73 = 1;
  reg [33-1:0] _tmp_74;
  reg _tmp_75;
  wire [32-1:0] _tmp_data_76;
  wire _tmp_valid_76;
  wire _tmp_ready_76;
  assign _tmp_ready_76 = (_tmp_74 > 0) && !_tmp_75;
  reg _ram_a_cond_1_1;
  reg [10-1:0] _tmp_77;
  reg [32-1:0] _tmp_78;
  reg [32-1:0] _tmp_79;
  reg [32-1:0] _tmp_80;
  reg [32-1:0] _tmp_fsm_4;
  localparam _tmp_fsm_4_init = 0;
  reg [9-1:0] _tmp_81;
  reg _myaxi_cond_5_1;
  wire _tmp_82;
  wire _tmp_83;
  assign _tmp_82 = 1 && _tmp_ready_86;
  assign _tmp_83 = 1;
  reg [33-1:0] _tmp_84;
  reg _tmp_85;
  wire [32-1:0] _tmp_data_86;
  wire _tmp_valid_86;
  wire _tmp_ready_86;
  assign _tmp_ready_86 = (_tmp_84 > 0) && !_tmp_85;
  reg _ram_b_cond_1_1;
  reg [32-1:0] th_sequential;
  localparam th_sequential_init = 0;
  reg _th_sequential_called;
  reg signed [32-1:0] _th_sequential_size_8;
  reg signed [32-1:0] _th_sequential_offset_9;
  reg signed [32-1:0] _th_sequential_size_10;
  reg signed [32-1:0] _th_sequential_offset_11;
  reg signed [32-1:0] _th_sequential_sum_12;
  reg signed [32-1:0] _th_sequential_i_13;
  reg _tmp_87;
  reg _ram_a_cond_2_1;
  reg _ram_a_cond_3_1;
  reg _ram_a_cond_3_2;
  reg signed [32-1:0] _tmp_88;
  reg signed [32-1:0] _th_sequential_a_14;
  reg _tmp_89;
  reg _ram_b_cond_2_1;
  reg _ram_b_cond_3_1;
  reg _ram_b_cond_3_2;
  reg signed [32-1:0] _tmp_90;
  reg signed [32-1:0] _th_sequential_b_15;
  reg _ram_c_cond_1_1;
  reg [10-1:0] _tmp_91;
  reg [32-1:0] _tmp_92;
  reg [32-1:0] _tmp_93;
  reg [32-1:0] _tmp_94;
  reg [32-1:0] _tmp_fsm_5;
  localparam _tmp_fsm_5_init = 0;
  reg [9-1:0] _tmp_95;
  reg _myaxi_cond_6_1;
  reg _tmp_96;
  reg _tmp_97;
  wire _tmp_98;
  wire _tmp_99;
  assign _tmp_98 = 1 && _tmp_ready_109;
  assign _tmp_99 = 1;
  localparam _tmp_100 = 1;
  wire [_tmp_100-1:0] _tmp_101;
  assign _tmp_101 = (_tmp_98 || !_tmp_96) && (_tmp_99 || !_tmp_97);
  reg [_tmp_100-1:0] __tmp_101_1;
  wire [32-1:0] _tmp_102;
  reg [32-1:0] __tmp_102_1;
  assign _tmp_102 = (__tmp_101_1)? ram_c_0_rdata : __tmp_102_1;
  reg [33-1:0] _tmp_103;
  reg _tmp_104;
  reg _tmp_105;
  reg _tmp_106;
  reg _tmp_107;
  reg _tmp_108;
  wire [32-1:0] _tmp_data_109;
  wire _tmp_valid_109;
  wire _tmp_ready_109;
  assign _tmp_ready_109 = (_tmp_fsm_5 == 3) && ((_tmp_95 > 0) && (myaxi_wready || !myaxi_wvalid));
  reg _myaxi_cond_7_1;
  reg signed [32-1:0] _th_comp_size_16;
  reg signed [32-1:0] _th_comp_offset_stream_17;
  reg signed [32-1:0] _th_comp_offset_seq_18;
  reg signed [32-1:0] _th_comp_all_ok_19;
  reg _tmp_110;
  reg _ram_c_cond_2_1;
  reg _ram_c_cond_3_1;
  reg _ram_c_cond_3_2;
  reg signed [32-1:0] _tmp_111;
  reg signed [32-1:0] _th_comp_st_20;
  reg _tmp_112;
  reg _ram_c_cond_4_1;
  reg _ram_c_cond_5_1;
  reg _ram_c_cond_5_2;
  reg signed [32-1:0] _tmp_113;
  reg signed [32-1:0] _th_comp_sq_21;

  always @(posedge CLK) begin
    if(RST) begin
      myaxi_araddr <= 0;
      myaxi_arlen <= 0;
      myaxi_arvalid <= 0;
      _tmp_4 <= 0;
      _myaxi_cond_0_1 <= 0;
      _tmp_14 <= 0;
      _myaxi_cond_1_1 <= 0;
      myaxi_awaddr <= 0;
      myaxi_awlen <= 0;
      myaxi_awvalid <= 0;
      _tmp_52 <= 0;
      _myaxi_cond_2_1 <= 0;
      myaxi_wdata <= 0;
      myaxi_wvalid <= 0;
      myaxi_wlast <= 0;
      myaxi_wstrb <= 0;
      _tmp_65 <= 0;
      _myaxi_cond_3_1 <= 0;
      _tmp_71 <= 0;
      _myaxi_cond_4_1 <= 0;
      _tmp_81 <= 0;
      _myaxi_cond_5_1 <= 0;
      _tmp_95 <= 0;
      _myaxi_cond_6_1 <= 0;
      _tmp_108 <= 0;
      _myaxi_cond_7_1 <= 0;
    end else begin
      if(_myaxi_cond_0_1) begin
        myaxi_arvalid <= 0;
      end 
      if(_myaxi_cond_1_1) begin
        myaxi_arvalid <= 0;
      end 
      if(_myaxi_cond_2_1) begin
        myaxi_awvalid <= 0;
      end 
      if(_myaxi_cond_3_1) begin
        myaxi_wvalid <= 0;
        myaxi_wlast <= 0;
        _tmp_65 <= 0;
      end 
      if(_myaxi_cond_4_1) begin
        myaxi_arvalid <= 0;
      end 
      if(_myaxi_cond_5_1) begin
        myaxi_arvalid <= 0;
      end 
      if(_myaxi_cond_6_1) begin
        myaxi_awvalid <= 0;
      end 
      if(_myaxi_cond_7_1) begin
        myaxi_wvalid <= 0;
        myaxi_wlast <= 0;
        _tmp_108 <= 0;
      end 
      if((_tmp_fsm_0 == 1) && ((myaxi_arready || !myaxi_arvalid) && (_tmp_4 == 0))) begin
        myaxi_araddr <= _tmp_1;
        myaxi_arlen <= _tmp_2 - 1;
        myaxi_arvalid <= 1;
        _tmp_4 <= _tmp_2;
      end 
      _myaxi_cond_0_1 <= 1;
      if(myaxi_arvalid && !myaxi_arready) begin
        myaxi_arvalid <= myaxi_arvalid;
      end 
      if(myaxi_rready && myaxi_rvalid && (_tmp_4 > 0)) begin
        _tmp_4 <= _tmp_4 - 1;
      end 
      if((_tmp_fsm_1 == 1) && ((myaxi_arready || !myaxi_arvalid) && (_tmp_14 == 0))) begin
        myaxi_araddr <= _tmp_11;
        myaxi_arlen <= _tmp_12 - 1;
        myaxi_arvalid <= 1;
        _tmp_14 <= _tmp_12;
      end 
      _myaxi_cond_1_1 <= 1;
      if(myaxi_arvalid && !myaxi_arready) begin
        myaxi_arvalid <= myaxi_arvalid;
      end 
      if(myaxi_rready && myaxi_rvalid && (_tmp_14 > 0)) begin
        _tmp_14 <= _tmp_14 - 1;
      end 
      if((_tmp_fsm_2 == 1) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_52 == 0))) begin
        myaxi_awaddr <= _tmp_49;
        myaxi_awlen <= _tmp_50 - 1;
        myaxi_awvalid <= 1;
        _tmp_52 <= _tmp_50;
      end 
      if((_tmp_fsm_2 == 1) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_52 == 0)) && (_tmp_50 == 0)) begin
        myaxi_awvalid <= 0;
      end 
      _myaxi_cond_2_1 <= 1;
      if(myaxi_awvalid && !myaxi_awready) begin
        myaxi_awvalid <= myaxi_awvalid;
      end 
      if(_tmp_valid_66 && ((_tmp_fsm_2 == 3) && ((_tmp_52 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_52 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_52 > 0))) begin
        myaxi_wdata <= _tmp_data_66;
        myaxi_wvalid <= 1;
        myaxi_wlast <= 0;
        myaxi_wstrb <= { 4{ 1'd1 } };
        _tmp_52 <= _tmp_52 - 1;
      end 
      if(_tmp_valid_66 && ((_tmp_fsm_2 == 3) && ((_tmp_52 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_52 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_52 > 0)) && (_tmp_52 == 1)) begin
        myaxi_wlast <= 1;
        _tmp_65 <= 1;
      end 
      _myaxi_cond_3_1 <= 1;
      if(myaxi_wvalid && !myaxi_wready) begin
        myaxi_wvalid <= myaxi_wvalid;
        myaxi_wlast <= myaxi_wlast;
        _tmp_65 <= _tmp_65;
      end 
      if((_tmp_fsm_3 == 1) && ((myaxi_arready || !myaxi_arvalid) && (_tmp_71 == 0))) begin
        myaxi_araddr <= _tmp_68;
        myaxi_arlen <= _tmp_69 - 1;
        myaxi_arvalid <= 1;
        _tmp_71 <= _tmp_69;
      end 
      _myaxi_cond_4_1 <= 1;
      if(myaxi_arvalid && !myaxi_arready) begin
        myaxi_arvalid <= myaxi_arvalid;
      end 
      if(myaxi_rready && myaxi_rvalid && (_tmp_71 > 0)) begin
        _tmp_71 <= _tmp_71 - 1;
      end 
      if((_tmp_fsm_4 == 1) && ((myaxi_arready || !myaxi_arvalid) && (_tmp_81 == 0))) begin
        myaxi_araddr <= _tmp_78;
        myaxi_arlen <= _tmp_79 - 1;
        myaxi_arvalid <= 1;
        _tmp_81 <= _tmp_79;
      end 
      _myaxi_cond_5_1 <= 1;
      if(myaxi_arvalid && !myaxi_arready) begin
        myaxi_arvalid <= myaxi_arvalid;
      end 
      if(myaxi_rready && myaxi_rvalid && (_tmp_81 > 0)) begin
        _tmp_81 <= _tmp_81 - 1;
      end 
      if((_tmp_fsm_5 == 1) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_95 == 0))) begin
        myaxi_awaddr <= _tmp_92;
        myaxi_awlen <= _tmp_93 - 1;
        myaxi_awvalid <= 1;
        _tmp_95 <= _tmp_93;
      end 
      if((_tmp_fsm_5 == 1) && ((myaxi_awready || !myaxi_awvalid) && (_tmp_95 == 0)) && (_tmp_93 == 0)) begin
        myaxi_awvalid <= 0;
      end 
      _myaxi_cond_6_1 <= 1;
      if(myaxi_awvalid && !myaxi_awready) begin
        myaxi_awvalid <= myaxi_awvalid;
      end 
      if(_tmp_valid_109 && ((_tmp_fsm_5 == 3) && ((_tmp_95 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_95 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_95 > 0))) begin
        myaxi_wdata <= _tmp_data_109;
        myaxi_wvalid <= 1;
        myaxi_wlast <= 0;
        myaxi_wstrb <= { 4{ 1'd1 } };
        _tmp_95 <= _tmp_95 - 1;
      end 
      if(_tmp_valid_109 && ((_tmp_fsm_5 == 3) && ((_tmp_95 > 0) && (myaxi_wready || !myaxi_wvalid))) && ((_tmp_95 > 0) && (myaxi_wready || !myaxi_wvalid) && (_tmp_95 > 0)) && (_tmp_95 == 1)) begin
        myaxi_wlast <= 1;
        _tmp_108 <= 1;
      end 
      _myaxi_cond_7_1 <= 1;
      if(myaxi_wvalid && !myaxi_wready) begin
        myaxi_wvalid <= myaxi_wvalid;
        myaxi_wlast <= myaxi_wlast;
        _tmp_108 <= _tmp_108;
      end 
    end
  end

  assign _tmp_data_9 = myaxi_rdata;
  assign _tmp_valid_9 = myaxi_rvalid;
  assign _tmp_data_19 = myaxi_rdata;
  assign _tmp_valid_19 = myaxi_rvalid;
  assign _tmp_data_76 = myaxi_rdata;
  assign _tmp_valid_76 = myaxi_rvalid;
  assign _tmp_data_86 = myaxi_rdata;
  assign _tmp_valid_86 = myaxi_rvalid;

  always @(posedge CLK) begin
    if(RST) begin
      ram_a_0_addr <= 0;
      _tmp_7 <= 0;
      ram_a_0_wdata <= 0;
      ram_a_0_wenable <= 0;
      _tmp_8 <= 0;
      _ram_a_cond_0_1 <= 0;
      __tmp_25_1 <= 0;
      __tmp_26_1 <= 0;
      _tmp_31 <= 0;
      _tmp_20 <= 0;
      _tmp_21 <= 0;
      _tmp_29 <= 0;
      _tmp_30 <= 0;
      _tmp_28 <= 0;
      _tmp_27 <= 0;
      _tmp_74 <= 0;
      _tmp_75 <= 0;
      _ram_a_cond_1_1 <= 0;
      _ram_a_cond_2_1 <= 0;
      _tmp_87 <= 0;
      _ram_a_cond_3_1 <= 0;
      _ram_a_cond_3_2 <= 0;
    end else begin
      if(_ram_a_cond_3_2) begin
        _tmp_87 <= 0;
      end 
      if(_ram_a_cond_0_1) begin
        ram_a_0_wenable <= 0;
        _tmp_8 <= 0;
      end 
      if(_ram_a_cond_1_1) begin
        ram_a_0_wenable <= 0;
        _tmp_75 <= 0;
      end 
      if(_ram_a_cond_2_1) begin
        _tmp_87 <= 1;
      end 
      _ram_a_cond_3_2 <= _ram_a_cond_3_1;
      if((_tmp_fsm_0 == 2) && (_tmp_7 == 0)) begin
        ram_a_0_addr <= _tmp_0 - 1;
        _tmp_7 <= _tmp_2;
      end 
      if(_tmp_valid_9 && ((_tmp_7 > 0) && !_tmp_8) && (_tmp_7 > 0)) begin
        ram_a_0_addr <= ram_a_0_addr + 1;
        ram_a_0_wdata <= _tmp_data_9;
        ram_a_0_wenable <= 1;
        _tmp_7 <= _tmp_7 - 1;
      end 
      if(_tmp_valid_9 && ((_tmp_7 > 0) && !_tmp_8) && (_tmp_7 == 1)) begin
        _tmp_8 <= 1;
      end 
      _ram_a_cond_0_1 <= 1;
      __tmp_25_1 <= _tmp_25;
      __tmp_26_1 <= _tmp_26;
      if((_tmp_22 || !_tmp_20) && (_tmp_23 || !_tmp_21) && _tmp_29) begin
        _tmp_31 <= 0;
        _tmp_20 <= 0;
        _tmp_21 <= 0;
        _tmp_29 <= 0;
      end 
      if((_tmp_22 || !_tmp_20) && (_tmp_23 || !_tmp_21) && _tmp_28) begin
        _tmp_20 <= 1;
        _tmp_21 <= 1;
        _tmp_31 <= _tmp_30;
        _tmp_30 <= 0;
        _tmp_28 <= 0;
        _tmp_29 <= 1;
      end 
      if((_mystream_fsm_3 == 1) && (_tmp_27 == 0) && !_tmp_30 && !_tmp_31) begin
        ram_a_0_addr <= _th_comp_offset_1;
        _tmp_27 <= _th_comp_size_0 - 1;
        _tmp_28 <= 1;
      end 
      if((_tmp_22 || !_tmp_20) && (_tmp_23 || !_tmp_21) && (_tmp_27 > 0)) begin
        ram_a_0_addr <= ram_a_0_addr + 1;
        _tmp_27 <= _tmp_27 - 1;
        _tmp_28 <= 1;
        _tmp_30 <= 0;
      end 
      if((_tmp_22 || !_tmp_20) && (_tmp_23 || !_tmp_21) && (_tmp_27 == 1)) begin
        _tmp_30 <= 1;
      end 
      if((_tmp_fsm_3 == 2) && (_tmp_74 == 0)) begin
        ram_a_0_addr <= _tmp_67 - 1;
        _tmp_74 <= _tmp_69;
      end 
      if(_tmp_valid_76 && ((_tmp_74 > 0) && !_tmp_75) && (_tmp_74 > 0)) begin
        ram_a_0_addr <= ram_a_0_addr + 1;
        ram_a_0_wdata <= _tmp_data_76;
        ram_a_0_wenable <= 1;
        _tmp_74 <= _tmp_74 - 1;
      end 
      if(_tmp_valid_76 && ((_tmp_74 > 0) && !_tmp_75) && (_tmp_74 == 1)) begin
        _tmp_75 <= 1;
      end 
      _ram_a_cond_1_1 <= 1;
      if(th_sequential == 5) begin
        ram_a_0_addr <= _th_sequential_i_13 + _th_sequential_offset_11;
      end 
      _ram_a_cond_2_1 <= th_sequential == 5;
      _ram_a_cond_3_1 <= th_sequential == 5;
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      ram_b_0_addr <= 0;
      _tmp_17 <= 0;
      ram_b_0_wdata <= 0;
      ram_b_0_wenable <= 0;
      _tmp_18 <= 0;
      _ram_b_cond_0_1 <= 0;
      __tmp_37_1 <= 0;
      __tmp_38_1 <= 0;
      _tmp_43 <= 0;
      _tmp_32 <= 0;
      _tmp_33 <= 0;
      _tmp_41 <= 0;
      _tmp_42 <= 0;
      _tmp_40 <= 0;
      _tmp_39 <= 0;
      _tmp_84 <= 0;
      _tmp_85 <= 0;
      _ram_b_cond_1_1 <= 0;
      _ram_b_cond_2_1 <= 0;
      _tmp_89 <= 0;
      _ram_b_cond_3_1 <= 0;
      _ram_b_cond_3_2 <= 0;
    end else begin
      if(_ram_b_cond_3_2) begin
        _tmp_89 <= 0;
      end 
      if(_ram_b_cond_0_1) begin
        ram_b_0_wenable <= 0;
        _tmp_18 <= 0;
      end 
      if(_ram_b_cond_1_1) begin
        ram_b_0_wenable <= 0;
        _tmp_85 <= 0;
      end 
      if(_ram_b_cond_2_1) begin
        _tmp_89 <= 1;
      end 
      _ram_b_cond_3_2 <= _ram_b_cond_3_1;
      if((_tmp_fsm_1 == 2) && (_tmp_17 == 0)) begin
        ram_b_0_addr <= _tmp_10 - 1;
        _tmp_17 <= _tmp_12;
      end 
      if(_tmp_valid_19 && ((_tmp_17 > 0) && !_tmp_18) && (_tmp_17 > 0)) begin
        ram_b_0_addr <= ram_b_0_addr + 1;
        ram_b_0_wdata <= _tmp_data_19;
        ram_b_0_wenable <= 1;
        _tmp_17 <= _tmp_17 - 1;
      end 
      if(_tmp_valid_19 && ((_tmp_17 > 0) && !_tmp_18) && (_tmp_17 == 1)) begin
        _tmp_18 <= 1;
      end 
      _ram_b_cond_0_1 <= 1;
      __tmp_37_1 <= _tmp_37;
      __tmp_38_1 <= _tmp_38;
      if((_tmp_34 || !_tmp_32) && (_tmp_35 || !_tmp_33) && _tmp_41) begin
        _tmp_43 <= 0;
        _tmp_32 <= 0;
        _tmp_33 <= 0;
        _tmp_41 <= 0;
      end 
      if((_tmp_34 || !_tmp_32) && (_tmp_35 || !_tmp_33) && _tmp_40) begin
        _tmp_32 <= 1;
        _tmp_33 <= 1;
        _tmp_43 <= _tmp_42;
        _tmp_42 <= 0;
        _tmp_40 <= 0;
        _tmp_41 <= 1;
      end 
      if((_mystream_fsm_5 == 1) && (_tmp_39 == 0) && !_tmp_42 && !_tmp_43) begin
        ram_b_0_addr <= _th_comp_offset_1;
        _tmp_39 <= _th_comp_size_0 - 1;
        _tmp_40 <= 1;
      end 
      if((_tmp_34 || !_tmp_32) && (_tmp_35 || !_tmp_33) && (_tmp_39 > 0)) begin
        ram_b_0_addr <= ram_b_0_addr + 1;
        _tmp_39 <= _tmp_39 - 1;
        _tmp_40 <= 1;
        _tmp_42 <= 0;
      end 
      if((_tmp_34 || !_tmp_32) && (_tmp_35 || !_tmp_33) && (_tmp_39 == 1)) begin
        _tmp_42 <= 1;
      end 
      if((_tmp_fsm_4 == 2) && (_tmp_84 == 0)) begin
        ram_b_0_addr <= _tmp_77 - 1;
        _tmp_84 <= _tmp_79;
      end 
      if(_tmp_valid_86 && ((_tmp_84 > 0) && !_tmp_85) && (_tmp_84 > 0)) begin
        ram_b_0_addr <= ram_b_0_addr + 1;
        ram_b_0_wdata <= _tmp_data_86;
        ram_b_0_wenable <= 1;
        _tmp_84 <= _tmp_84 - 1;
      end 
      if(_tmp_valid_86 && ((_tmp_84 > 0) && !_tmp_85) && (_tmp_84 == 1)) begin
        _tmp_85 <= 1;
      end 
      _ram_b_cond_1_1 <= 1;
      if(th_sequential == 7) begin
        ram_b_0_addr <= _th_sequential_i_13 + _th_sequential_offset_11;
      end 
      _ram_b_cond_2_1 <= th_sequential == 7;
      _ram_b_cond_3_1 <= th_sequential == 7;
    end
  end


  always @(posedge CLK) begin
    if(RST) begin
      ram_c_0_addr <= 0;
      _tmp_44 <= 0;
      ram_c_0_wdata <= 0;
      ram_c_0_wenable <= 0;
      _tmp_45 <= 0;
      _ram_c_cond_0_1 <= 0;
      __tmp_58_1 <= 0;
      __tmp_59_1 <= 0;
      _tmp_64 <= 0;
      _tmp_53 <= 0;
      _tmp_54 <= 0;
      _tmp_62 <= 0;
      _tmp_63 <= 0;
      _tmp_61 <= 0;
      _tmp_60 <= 0;
      _ram_c_cond_1_1 <= 0;
      __tmp_101_1 <= 0;
      __tmp_102_1 <= 0;
      _tmp_107 <= 0;
      _tmp_96 <= 0;
      _tmp_97 <= 0;
      _tmp_105 <= 0;
      _tmp_106 <= 0;
      _tmp_104 <= 0;
      _tmp_103 <= 0;
      _ram_c_cond_2_1 <= 0;
      _tmp_110 <= 0;
      _ram_c_cond_3_1 <= 0;
      _ram_c_cond_3_2 <= 0;
      _ram_c_cond_4_1 <= 0;
      _tmp_112 <= 0;
      _ram_c_cond_5_1 <= 0;
      _ram_c_cond_5_2 <= 0;
    end else begin
      if(_ram_c_cond_3_2) begin
        _tmp_110 <= 0;
      end 
      if(_ram_c_cond_5_2) begin
        _tmp_112 <= 0;
      end 
      if(_ram_c_cond_0_1) begin
        ram_c_0_wenable <= 0;
        _tmp_45 <= 0;
      end 
      if(_ram_c_cond_1_1) begin
        ram_c_0_wenable <= 0;
      end 
      if(_ram_c_cond_2_1) begin
        _tmp_110 <= 1;
      end 
      _ram_c_cond_3_2 <= _ram_c_cond_3_1;
      if(_ram_c_cond_4_1) begin
        _tmp_112 <= 1;
      end 
      _ram_c_cond_5_2 <= _ram_c_cond_5_1;
      if((_mystream_fsm_7 == 1) && (_tmp_44 == 0)) begin
        ram_c_0_addr <= _th_comp_offset_1 - 1;
        _tmp_44 <= 1;
      end 
      if(_tmp_data_47 && (_tmp_valid_47 && ((_tmp_44 > 0) && !_tmp_45)) && (_tmp_valid_46 && ((_tmp_44 > 0) && !_tmp_45)) && (_tmp_44 > 0)) begin
        ram_c_0_addr <= ram_c_0_addr + 1;
        ram_c_0_wdata <= _tmp_data_46;
        ram_c_0_wenable <= 1;
        _tmp_44 <= _tmp_44 - 1;
      end 
      if(_tmp_data_47 && (_tmp_valid_47 && ((_tmp_44 > 0) && !_tmp_45)) && (_tmp_valid_46 && ((_tmp_44 > 0) && !_tmp_45)) && (_tmp_44 == 1)) begin
        _tmp_45 <= 1;
      end 
      _ram_c_cond_0_1 <= 1;
      __tmp_58_1 <= _tmp_58;
      __tmp_59_1 <= _tmp_59;
      if((_tmp_55 || !_tmp_53) && (_tmp_56 || !_tmp_54) && _tmp_62) begin
        _tmp_64 <= 0;
        _tmp_53 <= 0;
        _tmp_54 <= 0;
        _tmp_62 <= 0;
      end 
      if((_tmp_55 || !_tmp_53) && (_tmp_56 || !_tmp_54) && _tmp_61) begin
        _tmp_53 <= 1;
        _tmp_54 <= 1;
        _tmp_64 <= _tmp_63;
        _tmp_63 <= 0;
        _tmp_61 <= 0;
        _tmp_62 <= 1;
      end 
      if((_tmp_fsm_2 == 2) && (_tmp_60 == 0) && !_tmp_63 && !_tmp_64) begin
        ram_c_0_addr <= _tmp_48;
        _tmp_60 <= _tmp_50 - 1;
        _tmp_61 <= 1;
      end 
      if((_tmp_55 || !_tmp_53) && (_tmp_56 || !_tmp_54) && (_tmp_60 > 0)) begin
        ram_c_0_addr <= ram_c_0_addr + 1;
        _tmp_60 <= _tmp_60 - 1;
        _tmp_61 <= 1;
        _tmp_63 <= 0;
      end 
      if((_tmp_55 || !_tmp_53) && (_tmp_56 || !_tmp_54) && (_tmp_60 == 1)) begin
        _tmp_63 <= 1;
      end 
      if(th_sequential == 11) begin
        ram_c_0_addr <= _th_sequential_offset_11;
        ram_c_0_wdata <= _th_sequential_sum_12;
        ram_c_0_wenable <= 1;
      end 
      _ram_c_cond_1_1 <= th_sequential == 11;
      __tmp_101_1 <= _tmp_101;
      __tmp_102_1 <= _tmp_102;
      if((_tmp_98 || !_tmp_96) && (_tmp_99 || !_tmp_97) && _tmp_105) begin
        _tmp_107 <= 0;
        _tmp_96 <= 0;
        _tmp_97 <= 0;
        _tmp_105 <= 0;
      end 
      if((_tmp_98 || !_tmp_96) && (_tmp_99 || !_tmp_97) && _tmp_104) begin
        _tmp_96 <= 1;
        _tmp_97 <= 1;
        _tmp_107 <= _tmp_106;
        _tmp_106 <= 0;
        _tmp_104 <= 0;
        _tmp_105 <= 1;
      end 
      if((_tmp_fsm_5 == 2) && (_tmp_103 == 0) && !_tmp_106 && !_tmp_107) begin
        ram_c_0_addr <= _tmp_91;
        _tmp_103 <= _tmp_93 - 1;
        _tmp_104 <= 1;
      end 
      if((_tmp_98 || !_tmp_96) && (_tmp_99 || !_tmp_97) && (_tmp_103 > 0)) begin
        ram_c_0_addr <= ram_c_0_addr + 1;
        _tmp_103 <= _tmp_103 - 1;
        _tmp_104 <= 1;
        _tmp_106 <= 0;
      end 
      if((_tmp_98 || !_tmp_96) && (_tmp_99 || !_tmp_97) && (_tmp_103 == 1)) begin
        _tmp_106 <= 1;
      end 
      if(th_comp == 27) begin
        ram_c_0_addr <= _th_comp_offset_stream_17;
      end 
      _ram_c_cond_2_1 <= th_comp == 27;
      _ram_c_cond_3_1 <= th_comp == 27;
      if(th_comp == 29) begin
        ram_c_0_addr <= _th_comp_offset_seq_18;
      end 
      _ram_c_cond_4_1 <= th_comp == 29;
      _ram_c_cond_5_1 <= th_comp == 29;
    end
  end

  assign _tmp_data_66 = _tmp_59;
  assign _tmp_valid_66 = _tmp_53;
  assign _tmp_data_109 = _tmp_102;
  assign _tmp_valid_109 = _tmp_96;
  reg [32-1:0] _tmp_data_114;
  reg _tmp_valid_114;
  wire _tmp_ready_114;
  assign _tmp_ready_114 = (_tmp_ready_116 || !_tmp_valid_116) && _tmp_valid_114;
  wire [32-1:0] _tmp_data_115;
  wire _tmp_valid_115;
  wire _tmp_ready_115;
  wire [64-1:0] _tmp_odata_115;
  reg [64-1:0] _tmp_data_reg_115;
  assign _tmp_data_115 = _tmp_data_reg_115;
  wire _tmp_ovalid_115;
  reg _tmp_valid_reg_115;
  assign _tmp_valid_115 = _tmp_valid_reg_115;
  wire _tmp_enable_115;
  wire _tmp_update_115;
  assign _tmp_enable_115 = (_tmp_ready_115 || !_tmp_valid_115) && (_tmp_22 && _tmp_34) && (_tmp_20 && _tmp_32);
  assign _tmp_update_115 = _tmp_ready_115 || !_tmp_valid_115;

  multiplier_0
  mul115
  (
    .CLK(CLK),
    .RST(RST),
    .update(_tmp_update_115),
    .enable(_tmp_enable_115),
    .valid(_tmp_ovalid_115),
    .a(_tmp_26),
    .b(_tmp_38),
    .c(_tmp_odata_115)
  );

  assign _tmp_ready_115 = (_tmp_ready_131 || !_tmp_valid_131) && (_tmp_valid_115 && _tmp_valid_129);
  reg [1-1:0] _tmp_data_116;
  reg _tmp_valid_116;
  wire _tmp_ready_116;
  assign _tmp_ready_116 = (_tmp_ready_119 || !_tmp_valid_119) && _tmp_valid_116 && ((_tmp_ready_120 || !_tmp_valid_120) && _tmp_valid_116);
  reg [1-1:0] _tmp_data_117;
  reg [1-1:0] _tmp_data_118;
  reg [1-1:0] _tmp_data_119;
  reg _tmp_valid_119;
  wire _tmp_ready_119;
  assign _tmp_ready_119 = (_tmp_ready_121 || !_tmp_valid_121) && _tmp_valid_119;
  reg [1-1:0] _tmp_data_120;
  reg _tmp_valid_120;
  wire _tmp_ready_120;
  assign _tmp_ready_120 = (_tmp_ready_122 || !_tmp_valid_122) && _tmp_valid_120;
  reg [1-1:0] _tmp_data_121;
  reg _tmp_valid_121;
  wire _tmp_ready_121;
  assign _tmp_ready_121 = (_tmp_ready_123 || !_tmp_valid_123) && _tmp_valid_121;
  reg [1-1:0] _tmp_data_122;
  reg _tmp_valid_122;
  wire _tmp_ready_122;
  assign _tmp_ready_122 = (_tmp_ready_124 || !_tmp_valid_124) && _tmp_valid_122;
  reg [1-1:0] _tmp_data_123;
  reg _tmp_valid_123;
  wire _tmp_ready_123;
  assign _tmp_ready_123 = (_tmp_ready_125 || !_tmp_valid_125) && _tmp_valid_123;
  reg [1-1:0] _tmp_data_124;
  reg _tmp_valid_124;
  wire _tmp_ready_124;
  assign _tmp_ready_124 = (_tmp_ready_126 || !_tmp_valid_126) && _tmp_valid_124;
  reg [1-1:0] _tmp_data_125;
  reg _tmp_valid_125;
  wire _tmp_ready_125;
  assign _tmp_ready_125 = (_tmp_ready_127 || !_tmp_valid_127) && _tmp_valid_125;
  reg [1-1:0] _tmp_data_126;
  reg _tmp_valid_126;
  wire _tmp_ready_126;
  assign _tmp_ready_126 = (_tmp_ready_128 || !_tmp_valid_128) && _tmp_valid_126;
  reg [1-1:0] _tmp_data_127;
  reg _tmp_valid_127;
  wire _tmp_ready_127;
  assign _tmp_ready_127 = (_tmp_ready_129 || !_tmp_valid_129) && _tmp_valid_127;
  reg [1-1:0] _tmp_data_128;
  reg _tmp_valid_128;
  wire _tmp_ready_128;
  assign _tmp_ready_128 = (_tmp_ready_130 || !_tmp_valid_130) && _tmp_valid_128;
  reg [1-1:0] _tmp_data_129;
  reg _tmp_valid_129;
  wire _tmp_ready_129;
  assign _tmp_ready_129 = (_tmp_ready_131 || !_tmp_valid_131) && (_tmp_valid_115 && _tmp_valid_129);
  reg [1-1:0] _tmp_data_130;
  reg _tmp_valid_130;
  wire _tmp_ready_130;
  assign _tmp_ready_130 = (_tmp_ready_132 || !_tmp_valid_132) && _tmp_valid_130;
  reg [32-1:0] _tmp_data_131;
  reg _tmp_valid_131;
  wire _tmp_ready_131;
  reg [1-1:0] _tmp_data_132;
  reg _tmp_valid_132;
  wire _tmp_ready_132;
  assign _tmp_data_46 = _tmp_data_131;
  assign _tmp_valid_46 = _tmp_valid_131;
  assign _tmp_ready_131 = _tmp_ready_46;
  assign _tmp_data_47 = _tmp_data_132;
  assign _tmp_valid_47 = _tmp_valid_132;
  assign _tmp_ready_132 = _tmp_ready_47;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_data_114 <= 1'd0;
      _tmp_valid_114 <= 0;
      _tmp_data_reg_115 <= 0;
      _tmp_valid_reg_115 <= 0;
      _tmp_data_116 <= 0;
      _tmp_valid_116 <= 0;
      _tmp_data_117 <= 0;
      _tmp_data_118 <= 0;
      _tmp_data_119 <= 0;
      _tmp_valid_119 <= 0;
      _tmp_data_120 <= 0;
      _tmp_valid_120 <= 0;
      _tmp_data_121 <= 0;
      _tmp_valid_121 <= 0;
      _tmp_data_122 <= 0;
      _tmp_valid_122 <= 0;
      _tmp_data_123 <= 0;
      _tmp_valid_123 <= 0;
      _tmp_data_124 <= 0;
      _tmp_valid_124 <= 0;
      _tmp_data_125 <= 0;
      _tmp_valid_125 <= 0;
      _tmp_data_126 <= 0;
      _tmp_valid_126 <= 0;
      _tmp_data_127 <= 0;
      _tmp_valid_127 <= 0;
      _tmp_data_128 <= 0;
      _tmp_valid_128 <= 0;
      _tmp_data_129 <= 0;
      _tmp_valid_129 <= 0;
      _tmp_data_130 <= 0;
      _tmp_valid_130 <= 0;
      _tmp_data_131 <= 1'd0;
      _tmp_valid_131 <= 0;
      _tmp_data_132 <= 0;
      _tmp_valid_132 <= 0;
    end else begin
      if((_tmp_ready_114 || !_tmp_valid_114) && 1 && 1) begin
        _tmp_data_114 <= (_tmp_data_114 >= _th_comp_size_0 - 1)? 0 : _tmp_data_114 + 2'd1;
      end 
      if(_tmp_valid_114 && _tmp_ready_114) begin
        _tmp_valid_114 <= 0;
      end 
      if((_tmp_ready_114 || !_tmp_valid_114) && 1) begin
        _tmp_valid_114 <= 1;
      end 
      if(_tmp_ready_115 || !_tmp_valid_115) begin
        _tmp_data_reg_115 <= _tmp_odata_115;
      end 
      if(_tmp_ready_115 || !_tmp_valid_115) begin
        _tmp_valid_reg_115 <= _tmp_ovalid_115;
      end 
      if((_tmp_ready_116 || !_tmp_valid_116) && _tmp_ready_114 && _tmp_valid_114) begin
        _tmp_data_116 <= _tmp_data_114 == _th_comp_size_0 - 1;
      end 
      if(_tmp_valid_116 && _tmp_ready_116) begin
        _tmp_valid_116 <= 0;
      end 
      if((_tmp_ready_116 || !_tmp_valid_116) && _tmp_ready_114) begin
        _tmp_valid_116 <= _tmp_valid_114;
      end 
      if(_tmp_valid_116 && _tmp_ready_116) begin
        _tmp_data_117 <= _tmp_data_116;
      end 
      if(_tmp_valid_116 && _tmp_ready_116) begin
        _tmp_data_118 <= _tmp_data_117;
      end 
      if((_tmp_ready_119 || !_tmp_valid_119) && _tmp_ready_116 && _tmp_valid_116) begin
        _tmp_data_119 <= _tmp_data_118;
      end 
      if(_tmp_valid_119 && _tmp_ready_119) begin
        _tmp_valid_119 <= 0;
      end 
      if((_tmp_ready_119 || !_tmp_valid_119) && _tmp_ready_116) begin
        _tmp_valid_119 <= _tmp_valid_116;
      end 
      if((_tmp_ready_120 || !_tmp_valid_120) && _tmp_ready_116 && _tmp_valid_116) begin
        _tmp_data_120 <= _tmp_data_117;
      end 
      if(_tmp_valid_120 && _tmp_ready_120) begin
        _tmp_valid_120 <= 0;
      end 
      if((_tmp_ready_120 || !_tmp_valid_120) && _tmp_ready_116) begin
        _tmp_valid_120 <= _tmp_valid_116;
      end 
      if((_tmp_ready_121 || !_tmp_valid_121) && _tmp_ready_119 && _tmp_valid_119) begin
        _tmp_data_121 <= _tmp_data_119;
      end 
      if(_tmp_valid_121 && _tmp_ready_121) begin
        _tmp_valid_121 <= 0;
      end 
      if((_tmp_ready_121 || !_tmp_valid_121) && _tmp_ready_119) begin
        _tmp_valid_121 <= _tmp_valid_119;
      end 
      if((_tmp_ready_122 || !_tmp_valid_122) && _tmp_ready_120 && _tmp_valid_120) begin
        _tmp_data_122 <= _tmp_data_120;
      end 
      if(_tmp_valid_122 && _tmp_ready_122) begin
        _tmp_valid_122 <= 0;
      end 
      if((_tmp_ready_122 || !_tmp_valid_122) && _tmp_ready_120) begin
        _tmp_valid_122 <= _tmp_valid_120;
      end 
      if((_tmp_ready_123 || !_tmp_valid_123) && _tmp_ready_121 && _tmp_valid_121) begin
        _tmp_data_123 <= _tmp_data_121;
      end 
      if(_tmp_valid_123 && _tmp_ready_123) begin
        _tmp_valid_123 <= 0;
      end 
      if((_tmp_ready_123 || !_tmp_valid_123) && _tmp_ready_121) begin
        _tmp_valid_123 <= _tmp_valid_121;
      end 
      if((_tmp_ready_124 || !_tmp_valid_124) && _tmp_ready_122 && _tmp_valid_122) begin
        _tmp_data_124 <= _tmp_data_122;
      end 
      if(_tmp_valid_124 && _tmp_ready_124) begin
        _tmp_valid_124 <= 0;
      end 
      if((_tmp_ready_124 || !_tmp_valid_124) && _tmp_ready_122) begin
        _tmp_valid_124 <= _tmp_valid_122;
      end 
      if((_tmp_ready_125 || !_tmp_valid_125) && _tmp_ready_123 && _tmp_valid_123) begin
        _tmp_data_125 <= _tmp_data_123;
      end 
      if(_tmp_valid_125 && _tmp_ready_125) begin
        _tmp_valid_125 <= 0;
      end 
      if((_tmp_ready_125 || !_tmp_valid_125) && _tmp_ready_123) begin
        _tmp_valid_125 <= _tmp_valid_123;
      end 
      if((_tmp_ready_126 || !_tmp_valid_126) && _tmp_ready_124 && _tmp_valid_124) begin
        _tmp_data_126 <= _tmp_data_124;
      end 
      if(_tmp_valid_126 && _tmp_ready_126) begin
        _tmp_valid_126 <= 0;
      end 
      if((_tmp_ready_126 || !_tmp_valid_126) && _tmp_ready_124) begin
        _tmp_valid_126 <= _tmp_valid_124;
      end 
      if((_tmp_ready_127 || !_tmp_valid_127) && _tmp_ready_125 && _tmp_valid_125) begin
        _tmp_data_127 <= _tmp_data_125;
      end 
      if(_tmp_valid_127 && _tmp_ready_127) begin
        _tmp_valid_127 <= 0;
      end 
      if((_tmp_ready_127 || !_tmp_valid_127) && _tmp_ready_125) begin
        _tmp_valid_127 <= _tmp_valid_125;
      end 
      if((_tmp_ready_128 || !_tmp_valid_128) && _tmp_ready_126 && _tmp_valid_126) begin
        _tmp_data_128 <= _tmp_data_126;
      end 
      if(_tmp_valid_128 && _tmp_ready_128) begin
        _tmp_valid_128 <= 0;
      end 
      if((_tmp_ready_128 || !_tmp_valid_128) && _tmp_ready_126) begin
        _tmp_valid_128 <= _tmp_valid_126;
      end 
      if((_tmp_ready_129 || !_tmp_valid_129) && _tmp_ready_127 && _tmp_valid_127) begin
        _tmp_data_129 <= _tmp_data_127;
      end 
      if(_tmp_valid_129 && _tmp_ready_129) begin
        _tmp_valid_129 <= 0;
      end 
      if((_tmp_ready_129 || !_tmp_valid_129) && _tmp_ready_127) begin
        _tmp_valid_129 <= _tmp_valid_127;
      end 
      if((_tmp_ready_130 || !_tmp_valid_130) && _tmp_ready_128 && _tmp_valid_128) begin
        _tmp_data_130 <= _tmp_data_128;
      end 
      if(_tmp_valid_130 && _tmp_ready_130) begin
        _tmp_valid_130 <= 0;
      end 
      if((_tmp_ready_130 || !_tmp_valid_130) && _tmp_ready_128) begin
        _tmp_valid_130 <= _tmp_valid_128;
      end 
      if((_tmp_ready_131 || !_tmp_valid_131) && (_tmp_ready_115 && _tmp_ready_129) && (_tmp_valid_115 && _tmp_valid_129)) begin
        _tmp_data_131 <= _tmp_data_131 + _tmp_data_115;
      end 
      if(_tmp_valid_131 && _tmp_ready_131) begin
        _tmp_valid_131 <= 0;
      end 
      if((_tmp_ready_131 || !_tmp_valid_131) && (_tmp_ready_115 && _tmp_ready_129)) begin
        _tmp_valid_131 <= _tmp_valid_115 && _tmp_valid_129;
      end 
      if((_tmp_ready_131 || !_tmp_valid_131) && (_tmp_ready_115 && _tmp_ready_129) && (_tmp_valid_115 && _tmp_valid_129) && _tmp_data_129) begin
        _tmp_data_131 <= 1'd0 + _tmp_data_115;
      end 
      if((_tmp_ready_132 || !_tmp_valid_132) && _tmp_ready_130 && _tmp_valid_130) begin
        _tmp_data_132 <= _tmp_data_130;
      end 
      if(_tmp_valid_132 && _tmp_ready_132) begin
        _tmp_valid_132 <= 0;
      end 
      if((_tmp_ready_132 || !_tmp_valid_132) && _tmp_ready_130) begin
        _tmp_valid_132 <= _tmp_valid_130;
      end 
    end
  end

  localparam th_comp_1 = 1;
  localparam th_comp_2 = 2;
  localparam th_comp_3 = 3;
  localparam th_comp_4 = 4;
  localparam th_comp_5 = 5;
  localparam th_comp_6 = 6;
  localparam th_comp_7 = 7;
  localparam th_comp_8 = 8;
  localparam th_comp_9 = 9;
  localparam th_comp_10 = 10;
  localparam th_comp_11 = 11;
  localparam th_comp_12 = 12;
  localparam th_comp_13 = 13;
  localparam th_comp_14 = 14;
  localparam th_comp_15 = 15;
  localparam th_comp_16 = 16;
  localparam th_comp_17 = 17;
  localparam th_comp_18 = 18;
  localparam th_comp_19 = 19;
  localparam th_comp_20 = 20;
  localparam th_comp_21 = 21;
  localparam th_comp_22 = 22;
  localparam th_comp_23 = 23;
  localparam th_comp_24 = 24;
  localparam th_comp_25 = 25;
  localparam th_comp_26 = 26;
  localparam th_comp_27 = 27;
  localparam th_comp_28 = 28;
  localparam th_comp_29 = 29;
  localparam th_comp_30 = 30;
  localparam th_comp_31 = 31;
  localparam th_comp_32 = 32;
  localparam th_comp_33 = 33;
  localparam th_comp_34 = 34;
  localparam th_comp_35 = 35;
  localparam th_comp_36 = 36;
  localparam th_comp_37 = 37;

  always @(posedge CLK) begin
    if(RST) begin
      th_comp <= th_comp_init;
      _th_comp_size_0 <= 0;
      _th_comp_offset_1 <= 0;
      _tmp_0 <= 0;
      _tmp_1 <= 0;
      _tmp_3 <= 0;
      _tmp_2 <= 0;
      _tmp_10 <= 0;
      _tmp_11 <= 0;
      _tmp_13 <= 0;
      _tmp_12 <= 0;
      _tmp_48 <= 0;
      _tmp_49 <= 0;
      _tmp_51 <= 0;
      _tmp_50 <= 0;
      _tmp_67 <= 0;
      _tmp_68 <= 0;
      _tmp_70 <= 0;
      _tmp_69 <= 0;
      _tmp_77 <= 0;
      _tmp_78 <= 0;
      _tmp_80 <= 0;
      _tmp_79 <= 0;
      _tmp_91 <= 0;
      _tmp_92 <= 0;
      _tmp_94 <= 0;
      _tmp_93 <= 0;
      _th_comp_size_16 <= 0;
      _th_comp_offset_stream_17 <= 0;
      _th_comp_offset_seq_18 <= 0;
      _th_comp_all_ok_19 <= 0;
      _tmp_111 <= 0;
      _th_comp_st_20 <= 0;
      _tmp_113 <= 0;
      _th_comp_sq_21 <= 0;
    end else begin
      case(th_comp)
        th_comp_init: begin
          _th_comp_size_0 <= 32;
          th_comp <= th_comp_1;
        end
        th_comp_1: begin
          _th_comp_offset_1 <= 0;
          th_comp <= th_comp_2;
        end
        th_comp_2: begin
          _tmp_0 <= _th_comp_offset_1;
          _tmp_1 <= 0;
          _tmp_3 <= _th_comp_size_0;
          th_comp <= th_comp_3;
        end
        th_comp_3: begin
          if(_tmp_3 <= 256) begin
            _tmp_2 <= _tmp_3;
            _tmp_3 <= 0;
          end else begin
            _tmp_2 <= 256;
            _tmp_3 <= _tmp_3 - 256;
          end
          th_comp <= th_comp_4;
        end
        th_comp_4: begin
          if(_tmp_8) begin
            _tmp_0 <= _tmp_0 + _tmp_2;
            _tmp_1 <= _tmp_1 + (_tmp_2 << 2);
          end 
          if(_tmp_8 && (_tmp_3 > 0)) begin
            th_comp <= th_comp_3;
          end 
          if(_tmp_8 && (_tmp_3 == 0)) begin
            th_comp <= th_comp_5;
          end 
        end
        th_comp_5: begin
          _tmp_10 <= _th_comp_offset_1;
          _tmp_11 <= 0;
          _tmp_13 <= _th_comp_size_0;
          th_comp <= th_comp_6;
        end
        th_comp_6: begin
          if(_tmp_13 <= 256) begin
            _tmp_12 <= _tmp_13;
            _tmp_13 <= 0;
          end else begin
            _tmp_12 <= 256;
            _tmp_13 <= _tmp_13 - 256;
          end
          th_comp <= th_comp_7;
        end
        th_comp_7: begin
          if(_tmp_18) begin
            _tmp_10 <= _tmp_10 + _tmp_12;
            _tmp_11 <= _tmp_11 + (_tmp_12 << 2);
          end 
          if(_tmp_18 && (_tmp_13 > 0)) begin
            th_comp <= th_comp_6;
          end 
          if(_tmp_18 && (_tmp_13 == 0)) begin
            th_comp <= th_comp_8;
          end 
        end
        th_comp_8: begin
          th_comp <= th_comp_9;
        end
        th_comp_9: begin
          if(_mystream_flag_2 && _mystream_flag_4 && _mystream_flag_6) begin
            th_comp <= th_comp_10;
          end 
        end
        th_comp_10: begin
          _tmp_48 <= _th_comp_offset_1;
          _tmp_49 <= 1024;
          _tmp_51 <= 1;
          th_comp <= th_comp_11;
        end
        th_comp_11: begin
          if(_tmp_51 <= 256) begin
            _tmp_50 <= _tmp_51;
            _tmp_51 <= 0;
          end else begin
            _tmp_50 <= 256;
            _tmp_51 <= _tmp_51 - 256;
          end
          th_comp <= th_comp_12;
        end
        th_comp_12: begin
          if(_tmp_65) begin
            _tmp_48 <= _tmp_48 + _tmp_50;
            _tmp_49 <= _tmp_49 + (_tmp_50 << 2);
          end 
          if(_tmp_65 && (_tmp_51 > 0)) begin
            th_comp <= th_comp_11;
          end 
          if(_tmp_65 && (_tmp_51 == 0)) begin
            th_comp <= th_comp_13;
          end 
        end
        th_comp_13: begin
          _th_comp_offset_1 <= _th_comp_size_0;
          th_comp <= th_comp_14;
        end
        th_comp_14: begin
          _tmp_67 <= _th_comp_offset_1;
          _tmp_68 <= 0;
          _tmp_70 <= _th_comp_size_0;
          th_comp <= th_comp_15;
        end
        th_comp_15: begin
          if(_tmp_70 <= 256) begin
            _tmp_69 <= _tmp_70;
            _tmp_70 <= 0;
          end else begin
            _tmp_69 <= 256;
            _tmp_70 <= _tmp_70 - 256;
          end
          th_comp <= th_comp_16;
        end
        th_comp_16: begin
          if(_tmp_75) begin
            _tmp_67 <= _tmp_67 + _tmp_69;
            _tmp_68 <= _tmp_68 + (_tmp_69 << 2);
          end 
          if(_tmp_75 && (_tmp_70 > 0)) begin
            th_comp <= th_comp_15;
          end 
          if(_tmp_75 && (_tmp_70 == 0)) begin
            th_comp <= th_comp_17;
          end 
        end
        th_comp_17: begin
          _tmp_77 <= _th_comp_offset_1;
          _tmp_78 <= 0;
          _tmp_80 <= _th_comp_size_0;
          th_comp <= th_comp_18;
        end
        th_comp_18: begin
          if(_tmp_80 <= 256) begin
            _tmp_79 <= _tmp_80;
            _tmp_80 <= 0;
          end else begin
            _tmp_79 <= 256;
            _tmp_80 <= _tmp_80 - 256;
          end
          th_comp <= th_comp_19;
        end
        th_comp_19: begin
          if(_tmp_85) begin
            _tmp_77 <= _tmp_77 + _tmp_79;
            _tmp_78 <= _tmp_78 + (_tmp_79 << 2);
          end 
          if(_tmp_85 && (_tmp_80 > 0)) begin
            th_comp <= th_comp_18;
          end 
          if(_tmp_85 && (_tmp_80 == 0)) begin
            th_comp <= th_comp_20;
          end 
        end
        th_comp_20: begin
          th_comp <= th_comp_21;
        end
        th_comp_21: begin
          if(th_sequential == 12) begin
            th_comp <= th_comp_22;
          end 
        end
        th_comp_22: begin
          _tmp_91 <= _th_comp_offset_1;
          _tmp_92 <= 2048;
          _tmp_94 <= 1;
          th_comp <= th_comp_23;
        end
        th_comp_23: begin
          if(_tmp_94 <= 256) begin
            _tmp_93 <= _tmp_94;
            _tmp_94 <= 0;
          end else begin
            _tmp_93 <= 256;
            _tmp_94 <= _tmp_94 - 256;
          end
          th_comp <= th_comp_24;
        end
        th_comp_24: begin
          if(_tmp_108) begin
            _tmp_91 <= _tmp_91 + _tmp_93;
            _tmp_92 <= _tmp_92 + (_tmp_93 << 2);
          end 
          if(_tmp_108 && (_tmp_94 > 0)) begin
            th_comp <= th_comp_23;
          end 
          if(_tmp_108 && (_tmp_94 == 0)) begin
            th_comp <= th_comp_25;
          end 
        end
        th_comp_25: begin
          _th_comp_size_16 <= _th_comp_size_0;
          _th_comp_offset_stream_17 <= 0;
          _th_comp_offset_seq_18 <= _th_comp_offset_1;
          th_comp <= th_comp_26;
        end
        th_comp_26: begin
          _th_comp_all_ok_19 <= 1;
          th_comp <= th_comp_27;
        end
        th_comp_27: begin
          if(_tmp_110) begin
            _tmp_111 <= ram_c_0_rdata;
          end 
          if(_tmp_110) begin
            th_comp <= th_comp_28;
          end 
        end
        th_comp_28: begin
          _th_comp_st_20 <= _tmp_111;
          th_comp <= th_comp_29;
        end
        th_comp_29: begin
          if(_tmp_112) begin
            _tmp_113 <= ram_c_0_rdata;
          end 
          if(_tmp_112) begin
            th_comp <= th_comp_30;
          end 
        end
        th_comp_30: begin
          _th_comp_sq_21 <= _tmp_113;
          th_comp <= th_comp_31;
        end
        th_comp_31: begin
          if(_th_comp_st_20 != _th_comp_sq_21) begin
            th_comp <= th_comp_32;
          end else begin
            th_comp <= th_comp_33;
          end
        end
        th_comp_32: begin
          _th_comp_all_ok_19 <= 0;
          th_comp <= th_comp_33;
        end
        th_comp_33: begin
          if(_th_comp_all_ok_19) begin
            th_comp <= th_comp_34;
          end else begin
            th_comp <= th_comp_36;
          end
        end
        th_comp_34: begin
          $display("OK");
          th_comp <= th_comp_35;
        end
        th_comp_35: begin
          th_comp <= th_comp_37;
        end
        th_comp_36: begin
          $display("NG");
          th_comp <= th_comp_37;
        end
      endcase
    end
  end

  localparam _tmp_fsm_0_1 = 1;
  localparam _tmp_fsm_0_2 = 2;
  localparam _tmp_fsm_0_3 = 3;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_0 <= _tmp_fsm_0_init;
    end else begin
      case(_tmp_fsm_0)
        _tmp_fsm_0_init: begin
          if(th_comp == 4) begin
            _tmp_fsm_0 <= _tmp_fsm_0_1;
          end 
        end
        _tmp_fsm_0_1: begin
          if(myaxi_arready || !myaxi_arvalid) begin
            _tmp_fsm_0 <= _tmp_fsm_0_2;
          end 
        end
        _tmp_fsm_0_2: begin
          _tmp_fsm_0 <= _tmp_fsm_0_3;
        end
        _tmp_fsm_0_3: begin
          if(_tmp_8) begin
            _tmp_fsm_0 <= _tmp_fsm_0_init;
          end 
        end
      endcase
    end
  end

  localparam _tmp_fsm_1_1 = 1;
  localparam _tmp_fsm_1_2 = 2;
  localparam _tmp_fsm_1_3 = 3;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_1 <= _tmp_fsm_1_init;
    end else begin
      case(_tmp_fsm_1)
        _tmp_fsm_1_init: begin
          if(th_comp == 7) begin
            _tmp_fsm_1 <= _tmp_fsm_1_1;
          end 
        end
        _tmp_fsm_1_1: begin
          if(myaxi_arready || !myaxi_arvalid) begin
            _tmp_fsm_1 <= _tmp_fsm_1_2;
          end 
        end
        _tmp_fsm_1_2: begin
          _tmp_fsm_1 <= _tmp_fsm_1_3;
        end
        _tmp_fsm_1_3: begin
          if(_tmp_18) begin
            _tmp_fsm_1 <= _tmp_fsm_1_init;
          end 
        end
      endcase
    end
  end

  localparam _mystream_fsm_3_1 = 1;
  localparam _mystream_fsm_3_2 = 2;

  always @(posedge CLK) begin
    if(RST) begin
      _mystream_fsm_3 <= _mystream_fsm_3_init;
      _mystream_flag_2 <= 0;
    end else begin
      case(_mystream_fsm_3)
        _mystream_fsm_3_init: begin
          if(th_comp == 8) begin
            _mystream_flag_2 <= 0;
          end 
          if(th_comp == 8) begin
            _mystream_fsm_3 <= _mystream_fsm_3_1;
          end 
        end
        _mystream_fsm_3_1: begin
          _mystream_fsm_3 <= _mystream_fsm_3_2;
        end
        _mystream_fsm_3_2: begin
          if(_tmp_31) begin
            _mystream_flag_2 <= 1;
          end 
          if(_tmp_31) begin
            _mystream_fsm_3 <= _mystream_fsm_3_init;
          end 
        end
      endcase
    end
  end

  localparam _mystream_fsm_5_1 = 1;
  localparam _mystream_fsm_5_2 = 2;

  always @(posedge CLK) begin
    if(RST) begin
      _mystream_fsm_5 <= _mystream_fsm_5_init;
      _mystream_flag_4 <= 0;
    end else begin
      case(_mystream_fsm_5)
        _mystream_fsm_5_init: begin
          if(th_comp == 8) begin
            _mystream_flag_4 <= 0;
          end 
          if(th_comp == 8) begin
            _mystream_fsm_5 <= _mystream_fsm_5_1;
          end 
        end
        _mystream_fsm_5_1: begin
          _mystream_fsm_5 <= _mystream_fsm_5_2;
        end
        _mystream_fsm_5_2: begin
          if(_tmp_43) begin
            _mystream_flag_4 <= 1;
          end 
          if(_tmp_43) begin
            _mystream_fsm_5 <= _mystream_fsm_5_init;
          end 
        end
      endcase
    end
  end

  localparam _mystream_fsm_7_1 = 1;
  localparam _mystream_fsm_7_2 = 2;

  always @(posedge CLK) begin
    if(RST) begin
      _mystream_fsm_7 <= _mystream_fsm_7_init;
      _mystream_flag_6 <= 0;
    end else begin
      case(_mystream_fsm_7)
        _mystream_fsm_7_init: begin
          if(th_comp == 8) begin
            _mystream_flag_6 <= 0;
          end 
          if(th_comp == 8) begin
            _mystream_fsm_7 <= _mystream_fsm_7_1;
          end 
        end
        _mystream_fsm_7_1: begin
          _mystream_fsm_7 <= _mystream_fsm_7_2;
        end
        _mystream_fsm_7_2: begin
          if(_tmp_45) begin
            _mystream_flag_6 <= 1;
          end 
          if(_tmp_45) begin
            _mystream_fsm_7 <= _mystream_fsm_7_init;
          end 
        end
      endcase
    end
  end

  localparam _tmp_fsm_2_1 = 1;
  localparam _tmp_fsm_2_2 = 2;
  localparam _tmp_fsm_2_3 = 3;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_2 <= _tmp_fsm_2_init;
    end else begin
      case(_tmp_fsm_2)
        _tmp_fsm_2_init: begin
          if(th_comp == 12) begin
            _tmp_fsm_2 <= _tmp_fsm_2_1;
          end 
        end
        _tmp_fsm_2_1: begin
          if(myaxi_awready || !myaxi_awvalid) begin
            _tmp_fsm_2 <= _tmp_fsm_2_2;
          end 
        end
        _tmp_fsm_2_2: begin
          _tmp_fsm_2 <= _tmp_fsm_2_3;
        end
        _tmp_fsm_2_3: begin
          if(_tmp_65) begin
            _tmp_fsm_2 <= _tmp_fsm_2_init;
          end 
        end
      endcase
    end
  end

  localparam _tmp_fsm_3_1 = 1;
  localparam _tmp_fsm_3_2 = 2;
  localparam _tmp_fsm_3_3 = 3;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_3 <= _tmp_fsm_3_init;
    end else begin
      case(_tmp_fsm_3)
        _tmp_fsm_3_init: begin
          if(th_comp == 16) begin
            _tmp_fsm_3 <= _tmp_fsm_3_1;
          end 
        end
        _tmp_fsm_3_1: begin
          if(myaxi_arready || !myaxi_arvalid) begin
            _tmp_fsm_3 <= _tmp_fsm_3_2;
          end 
        end
        _tmp_fsm_3_2: begin
          _tmp_fsm_3 <= _tmp_fsm_3_3;
        end
        _tmp_fsm_3_3: begin
          if(_tmp_75) begin
            _tmp_fsm_3 <= _tmp_fsm_3_init;
          end 
        end
      endcase
    end
  end

  localparam _tmp_fsm_4_1 = 1;
  localparam _tmp_fsm_4_2 = 2;
  localparam _tmp_fsm_4_3 = 3;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_4 <= _tmp_fsm_4_init;
    end else begin
      case(_tmp_fsm_4)
        _tmp_fsm_4_init: begin
          if(th_comp == 19) begin
            _tmp_fsm_4 <= _tmp_fsm_4_1;
          end 
        end
        _tmp_fsm_4_1: begin
          if(myaxi_arready || !myaxi_arvalid) begin
            _tmp_fsm_4 <= _tmp_fsm_4_2;
          end 
        end
        _tmp_fsm_4_2: begin
          _tmp_fsm_4 <= _tmp_fsm_4_3;
        end
        _tmp_fsm_4_3: begin
          if(_tmp_85) begin
            _tmp_fsm_4 <= _tmp_fsm_4_init;
          end 
        end
      endcase
    end
  end

  localparam th_sequential_1 = 1;
  localparam th_sequential_2 = 2;
  localparam th_sequential_3 = 3;
  localparam th_sequential_4 = 4;
  localparam th_sequential_5 = 5;
  localparam th_sequential_6 = 6;
  localparam th_sequential_7 = 7;
  localparam th_sequential_8 = 8;
  localparam th_sequential_9 = 9;
  localparam th_sequential_10 = 10;
  localparam th_sequential_11 = 11;
  localparam th_sequential_12 = 12;

  always @(posedge CLK) begin
    if(RST) begin
      th_sequential <= th_sequential_init;
      _th_sequential_called <= 0;
      _th_sequential_size_8 <= 0;
      _th_sequential_offset_9 <= 0;
      _th_sequential_size_10 <= 0;
      _th_sequential_offset_11 <= 0;
      _th_sequential_sum_12 <= 0;
      _th_sequential_i_13 <= 0;
      _tmp_88 <= 0;
      _th_sequential_a_14 <= 0;
      _tmp_90 <= 0;
      _th_sequential_b_15 <= 0;
    end else begin
      case(th_sequential)
        th_sequential_init: begin
          if(th_comp == 20) begin
            _th_sequential_called <= 1;
          end 
          if(th_comp == 20) begin
            _th_sequential_size_8 <= _th_comp_size_0;
          end 
          if(th_comp == 20) begin
            _th_sequential_offset_9 <= _th_comp_offset_1;
          end 
          if(th_comp == 20) begin
            th_sequential <= th_sequential_1;
          end 
        end
        th_sequential_1: begin
          _th_sequential_size_10 <= _th_sequential_size_8;
          _th_sequential_offset_11 <= _th_sequential_offset_9;
          th_sequential <= th_sequential_2;
        end
        th_sequential_2: begin
          _th_sequential_sum_12 <= 0;
          th_sequential <= th_sequential_3;
        end
        th_sequential_3: begin
          _th_sequential_i_13 <= 0;
          th_sequential <= th_sequential_4;
        end
        th_sequential_4: begin
          if(_th_sequential_i_13 < _th_sequential_size_10) begin
            th_sequential <= th_sequential_5;
          end else begin
            th_sequential <= th_sequential_11;
          end
        end
        th_sequential_5: begin
          if(_tmp_87) begin
            _tmp_88 <= ram_a_0_rdata;
          end 
          if(_tmp_87) begin
            th_sequential <= th_sequential_6;
          end 
        end
        th_sequential_6: begin
          _th_sequential_a_14 <= _tmp_88;
          th_sequential <= th_sequential_7;
        end
        th_sequential_7: begin
          if(_tmp_89) begin
            _tmp_90 <= ram_b_0_rdata;
          end 
          if(_tmp_89) begin
            th_sequential <= th_sequential_8;
          end 
        end
        th_sequential_8: begin
          _th_sequential_b_15 <= _tmp_90;
          th_sequential <= th_sequential_9;
        end
        th_sequential_9: begin
          _th_sequential_sum_12 <= _th_sequential_sum_12 + _th_sequential_a_14 * _th_sequential_b_15;
          th_sequential <= th_sequential_10;
        end
        th_sequential_10: begin
          _th_sequential_i_13 <= _th_sequential_i_13 + 1;
          th_sequential <= th_sequential_4;
        end
        th_sequential_11: begin
          th_sequential <= th_sequential_12;
        end
      endcase
    end
  end

  localparam _tmp_fsm_5_1 = 1;
  localparam _tmp_fsm_5_2 = 2;
  localparam _tmp_fsm_5_3 = 3;

  always @(posedge CLK) begin
    if(RST) begin
      _tmp_fsm_5 <= _tmp_fsm_5_init;
    end else begin
      case(_tmp_fsm_5)
        _tmp_fsm_5_init: begin
          if(th_comp == 24) begin
            _tmp_fsm_5 <= _tmp_fsm_5_1;
          end 
        end
        _tmp_fsm_5_1: begin
          if(myaxi_awready || !myaxi_awvalid) begin
            _tmp_fsm_5 <= _tmp_fsm_5_2;
          end 
        end
        _tmp_fsm_5_2: begin
          _tmp_fsm_5 <= _tmp_fsm_5_3;
        end
        _tmp_fsm_5_3: begin
          if(_tmp_108) begin
            _tmp_fsm_5 <= _tmp_fsm_5_init;
          end 
        end
      endcase
    end
  end


endmodule



module ram_a
(
  input CLK,
  input [10-1:0] ram_a_0_addr,
  output [32-1:0] ram_a_0_rdata,
  input [32-1:0] ram_a_0_wdata,
  input ram_a_0_wenable
);

  reg [10-1:0] ram_a_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(ram_a_0_wenable) begin
      mem[ram_a_0_addr] <= ram_a_0_wdata;
    end 
    ram_a_0_daddr <= ram_a_0_addr;
  end

  assign ram_a_0_rdata = mem[ram_a_0_daddr];

endmodule



module ram_b
(
  input CLK,
  input [10-1:0] ram_b_0_addr,
  output [32-1:0] ram_b_0_rdata,
  input [32-1:0] ram_b_0_wdata,
  input ram_b_0_wenable
);

  reg [10-1:0] ram_b_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(ram_b_0_wenable) begin
      mem[ram_b_0_addr] <= ram_b_0_wdata;
    end 
    ram_b_0_daddr <= ram_b_0_addr;
  end

  assign ram_b_0_rdata = mem[ram_b_0_daddr];

endmodule



module ram_c
(
  input CLK,
  input [10-1:0] ram_c_0_addr,
  output [32-1:0] ram_c_0_rdata,
  input [32-1:0] ram_c_0_wdata,
  input ram_c_0_wenable
);

  reg [10-1:0] ram_c_0_daddr;
  reg [32-1:0] mem [0:1024-1];

  always @(posedge CLK) begin
    if(ram_c_0_wenable) begin
      mem[ram_c_0_addr] <= ram_c_0_wdata;
    end 
    ram_c_0_daddr <= ram_c_0_addr;
  end

  assign ram_c_0_rdata = mem[ram_c_0_daddr];

endmodule



module multiplier_0
(
  input CLK,
  input RST,
  input update,
  input enable,
  output valid,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg valid_reg0;
  reg valid_reg1;
  reg valid_reg2;
  reg valid_reg3;
  reg valid_reg4;
  reg valid_reg5;
  assign valid = valid_reg5;

  always @(posedge CLK) begin
    if(RST) begin
      valid_reg0 <= 0;
      valid_reg1 <= 0;
      valid_reg2 <= 0;
      valid_reg3 <= 0;
      valid_reg4 <= 0;
      valid_reg5 <= 0;
    end else begin
      if(update) begin
        valid_reg0 <= enable;
        valid_reg1 <= valid_reg0;
        valid_reg2 <= valid_reg1;
        valid_reg3 <= valid_reg2;
        valid_reg4 <= valid_reg3;
        valid_reg5 <= valid_reg4;
      end 
    end
  end


  multiplier_core_0
  mult
  (
    .CLK(CLK),
    .update(update),
    .a(a),
    .b(b),
    .c(c)
  );


endmodule



module multiplier_core_0
(
  input CLK,
  input update,
  input [32-1:0] a,
  input [32-1:0] b,
  output [64-1:0] c
);

  reg [32-1:0] _a;
  reg [32-1:0] _b;
  reg signed [64-1:0] _tmpval0;
  reg signed [64-1:0] _tmpval1;
  reg signed [64-1:0] _tmpval2;
  reg signed [64-1:0] _tmpval3;
  reg signed [64-1:0] _tmpval4;
  wire signed [64-1:0] rslt;
  assign rslt = $signed({ 1'd0, _a }) * $signed({ 1'd0, _b });
  assign c = _tmpval4;

  always @(posedge CLK) begin
    if(update) begin
      _a <= a;
      _b <= b;
      _tmpval0 <= rslt;
      _tmpval1 <= _tmpval0;
      _tmpval2 <= _tmpval1;
      _tmpval3 <= _tmpval2;
      _tmpval4 <= _tmpval3;
    end 
  end


endmodule
"""


def test():
    veriloggen.reset()
    test_module = thread_stream_when.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
import collections
import functools

import veriloggen.core.vtypes as vtypes
from veriloggen.seq.subst_visitor import SubstDstVisitor
from veriloggen.seq.reset_visitor import ResetVisitor
from veriloggen.seq.seq import Seq, make_condition


_tmp_count = 0


def reset():
    global _tmp_count
    _tmp_count = 0


def _tmp_name(prefix='_tmp_fsm'):
    global _tmp_count
    v = _tmp_count
    _tmp_count += 1
    ret = '_'.join([prefix, str(v)])
    return ret


def TmpFSM(m, clk, rst, width=32, initname='init'):
    name = _tmp_name()
    return FSM(m, name, clk, rst, width, initname)


class FSM(vtypes.VeriloggenNode):
    """ Finite State Machine Generator """

    def __init__(self, m, name, clk, rst, width=32, initname='init', nohook=False):
        self.m = m
        self.name = name
        self.clk = clk
        self.rst = rst
        self.width = width
        self.state_count = 0
        self.state = self.m.Reg(name, width)  # set initval later

        self.mark = {}  # key:index
        self._set_mark(0, self.name + '_' + initname)
        self.state.initval = self._get_mark(0)

        self.body = collections.defaultdict(list)
        self.jump = collections.defaultdict(list)

        self.delay_amount = 0
        self.delayed_state = {}  # key:delay
        self.delayed_body = collections.defaultdict(
            functools.partial(collections.defaultdict, list))  # key:delay
        self.tmp_count = 0

        self.dst_var = collections.OrderedDict()
        self.dst_visitor = SubstDstVisitor()
        self.reset_visitor = ResetVisitor()

        self.seq = Seq(self.m, self.name + '_par', clk, rst, nohook=True)

        self.done = False

        self.last_cond = []
        self.last_kwargs = {}
        self.last_if_statement = None
        self.elif_cond = None
        self.next_kwargs = {}

        if not nohook:
            self.m.add_hook(self.make_always)

    #-------------------------------------------------------------------------
    def goto(self, dst, cond=None, else_dst=None):
        if cond is None and 'cond' in self.next_kwargs:
            cond = self.next_kwargs['cond']
        self._clear_next_kwargs()
        self._clear_last_if_statement()
        self._clear_last_cond()

        src = self.current
        return self._go(src, dst, cond, else_dst)

    def goto_init(self, cond=None):
        if cond is None and 'cond' in self.next_kwargs:
            cond = self.next_kwargs['cond']
        self._clear_next_kwargs()
        self._clear_last_if_statement()
        self._clear_last_cond()

        src = self.current
        dst = 0
        return self._go(src, dst, cond)

    def goto_next(self, cond=None):
        if cond is None and 'cond' in self.next_kwargs:
            cond = self.next_kwargs['cond']
        self._clear_next_kwargs()
        self._clear_last_if_statement()
        self._clear_last_cond()

        src = self.current
        dst = self.current + 1
        ret = self._go(src, dst, cond=cond)
        self.inc()
        return ret

    def goto_from(self, src, dst, cond=None, else_dst=None):
        if cond is None and 'cond' in self.next_kwargs:
            cond = self.next_kwargs['cond']
        self._clear_next_kwargs()
        self._clear_last_if_statement()
        self._clear_last_cond()

        return self._go(src, dst, cond, else_dst)

    def inc(self):
        self._set_index(None)

    #-------------------------------------------------------------------------
    def add(self, *statement, **kwargs):
        """ add new assignments """
        kwargs.update(self.next_kwargs)
        self.last_kwargs = kwargs
        self._clear_next_kwargs()

        # if there is no attributes, Elif object is reused.
        has_args = not (len(kwargs) == 0 or  # has no args
                        (len(kwargs) == 1 and 'cond' in kwargs))  # has only 'cond'

        if self.elif_cond is not None and not has_args:
            next_call = self.last_if_statement.Elif(self.elif_cond)
            next_call(*statement)
            self.last_if_statement = next_call
            self._add_dst_var(statement)
            self._clear_elif_cond()
            return self

        self._clear_last_if_statement()
        return self._add_statement(statement, **kwargs)

    #-------------------------------------------------------------------------
    def Prev(self, var, delay, initval=0):
        return self.seq.Prev(var, delay, initval)

    #-------------------------------------------------------------------------
    def If(self, *cond):
        self._clear_elif_cond()

        cond = make_condition(*cond)

        if cond is None:
            return self

        if 'cond' not in self.next_kwargs:
            self.next_kwargs['cond'] = cond
        else:
            self.next_kwargs['cond'] = vtypes.Ands(
                self.next_kwargs['cond'], cond)

        self.last_cond = [self.next_kwargs['cond']]

        return self

    def Else(self, *statement, **kwargs):
        self._clear_elif_cond()

        if len(self.last_cond) == 0:
            raise ValueError("No previous condition for Else.")

        old = self.last_cond.pop()
        self.last_cond.append(vtypes.Not(old))

        # if the true-statement has delay attributes,
        # Else statement is separated.
        if 'delay' in self.last_kwargs and self.last_kwargs['delay'] > 0:
            prev_cond = self.last_cond
            ret = self.Then()(*statement)
            self.last_cond = prev_cond
            return ret

        # if there is additional attribute, Else statement is separated.
        has_args = not (len(self.next_kwargs) == 0 or  # has no args
                        (len(self.next_kwargs) == 1 and 'cond' in kwargs))  # has only 'cond'

        if has_args:
            prev_cond = self.last_cond
            ret = self.Then()(*statement)
            self.last_cond = prev_cond
            return ret

        if not isinstance(self.last_if_statement, vtypes.If):
            raise ValueError("Last if-statement is not If")

        self.last_if_statement.Else(*statement)
        self._add_dst_var(statement)

        return self

    def Elif(self, *cond):
        if len(self.last_cond) == 0:
            raise ValueError("No previous condition for Else.")

        cond = make_condition(*cond)

        old = self.last_cond.pop()
        self.last_cond.append(vtypes.Not(old))
        self.last_cond.append(cond)

        # if the true-statement has delay attributes, Else statement is
        # separated.
        if 'delay' in self.last_kwargs and self.last_kwargs['delay'] > 0:
            prev_cond = self.last_cond
            ret = self.Then()
            self.last_cond = prev_cond
            return ret

        if not isinstance(self.last_if_statement, vtypes.If):
            raise ValueError("Last if-statement is not If")

        self.elif_cond = cond

        cond = self._make_cond(self.last_cond)
        self.next_kwargs['cond'] = cond

        return self

    def Delay(self, delay):
        self.next_kwargs['delay'] = delay
        return self

    def Keep(self, keep):
        self.next_kwargs['keep'] = keep
        return self

    def Then(self):
        cond = self._make_cond(self.last_cond)
        self._clear_last_cond()
        self.If(cond)
        return self

    def LazyCond(self, value=True):
        self.next_kwargs['lazy_cond'] = value
        return self

    def EagerVal(self, value=True):
        self.next_kwargs['eager_val'] = value
        return self

    def Clear(self):
        self._clear_next_kwargs()
        self._clear_last_if_statement()
        self._clear_last_cond()
        self._clear_elif_cond()
        return self

    #-------------------------------------------------------------------------
    @property
    def current(self):
        return self.state_count

    @property
    def next(self):
        return self.current + 1

    @property
    def current_delay(self):
        if 'delay' in self.next_kwargs:
            return self.next_kwargs['delay']
        return 0

    @property
    def last_delay(self):
        if 'delay' in self.last_kwargs:
            return self.last_kwargs['delay']
        return 0

    @property
    def current_condition(self):
        cond = self.next_kwargs['cond'] if 'cond' in self.next_kwargs else None
        if cond is not None:
            cond = vtypes.AndList(self.state == self.state_count, cond)
        else:
            cond = self.state == self.state_count
        return cond

    @property
    def last_condition(self):
        cond = self._make_cond(self.last_cond)
        if cond is not None:
            cond = vtypes.AndList(self.state == self.state_count, cond)
        else:
            cond = self.state == self.state_count
        return cond

    @property
    def then(self):
        return self.last_condition

    #-------------------------------------------------------------------------
    def make_always(self, reset=(), body=(), case=True):
        if self.done:
            #raise ValueError('make_always() has been already called.')
            return

        self.done = True

        part_reset = self.make_reset(reset)
        part_body = list(body) + list(self.make_case()
                                      if case else self.make_if())
        self.m.Always(vtypes.Posedge(self.clk))(
            vtypes.If(self.rst)(
                part_reset,
            )(
                part_body,
            ))

    #-------------------------------------------------------------------------
    def make_case(self):
        indexes = set(self.body.keys())
        indexes.update(set(self.jump.keys()))

        for index in indexes:
            self._add_mark(index)

        ret = []
        ret.extend(self.seq.make_code())
        ret.extend(self._get_delayed_substs())

        for delay, dct in sorted(self.delayed_body.items(),
                                 key=lambda x: x[0], reverse=True):
            body = tuple([self._get_delayed_when_statement(index, delay)
                          for index in sorted(dct.keys(), key=lambda x:x)])
            case = vtypes.Case(self._get_delayed_state(delay))(*body)
            ret.append(case)

        body = tuple([self._get_when_statement(index)
                      for index in sorted(indexes, key=lambda x:x)])
        case = vtypes.Case(self.state)(*body)
        ret.append(case)

        return ret

    def make_if(self):
        indexes = set(self.body.keys())
        indexes.update(set(self.jump.keys()))

        for index in indexes:
            self._add_mark(index)

        ret = []
        ret.extend(self.seq.make_code())
        ret.extend(self._get_delayed_substs())

        for delay, dct in sorted(self.delayed_body.items(),
                                 key=lambda x: x[0], reverse=True):
            ret.append([self._get_delayed_if_statement(index, delay)
                        for index in sorted(dct.keys(), key=lambda x:x)])

        ret.extend([self._get_if_statement(index)
                    for index in sorted(indexes, key=lambda x:x)])
        return ret

    #-------------------------------------------------------------------------
    def make_reset(self, reset):
        ret = collections.OrderedDict()

        for v in reset:
            if not isinstance(v, vtypes.Subst):
                raise TypeError(
                    'make_reset requires Subst, not %s' % str(type(v)))
            key = str(v.left)
            if key not in ret:
                ret[key] = v

        v = self.reset_visitor.visit(self.state)
        key = str(self.state)
        if v is not None and key not in ret:
            ret[key] = v

        for dst in self.delayed_state.values():
            v = self.reset_visitor.visit(dst)
            if v is None:
                continue
            key = str(v.left)
            if key not in ret:
                ret[key] = v

        for dst in self.dst_var.values():
            v = self.reset_visitor.visit(dst)
            if v is None:
                continue
            key = str(v.left)
            if key not in ret:
                ret[key] = v

        for v in self.seq.make_reset():
            if not isinstance(v, vtypes.Subst):
                raise TypeError(
                    'make_reset requires Subst, not %s' % str(type(v)))
            key = str(v.left)
            if key not in ret:
                ret[key] = v

        return list(ret.values())

    #-------------------------------------------------------------------------
    def _go(self, src, dst, cond=None, else_dst=None):
        self._add_jump(src, dst, cond, else_dst)
        return self

    def _add_jump(self, src, dst, cond=None, else_dst=None):
        self.jump[src].append((dst, cond, else_dst))

    #-------------------------------------------------------------------------
    def _add_statement(self, statement, index=None, keep=None, delay=None, cond=None,
                       lazy_cond=False, eager_val=False, no_delay_cond=False):

        cond = make_condition(cond)
        index = self._to_index(index) if index is not None else self.current

        if keep is not None:
            for i in range(keep):
                new_delay = i if delay is None else delay + i
                self._add_statement(statement, index=index,
                                    keep=None, delay=new_delay, cond=cond,
                                    lazy_cond=lazy_cond, eager_val=eager_val,
                                    no_delay_cond=no_delay_cond)
            return self

        if delay is not None and delay > 0:
            self._add_delayed_state(delay)

            if eager_val:
                statement = [self._add_delayed_subst(s, index, delay)
                             for s in statement]

            if not no_delay_cond:
                if cond is None:
                    cond = 1

                if not lazy_cond:
                    cond = self._add_delayed_cond(cond, index, delay)

                else:  # lazy condition
                    t = self._add_delayed_cond(1, index, delay)
                    if isinstance(cond, int) and cond == 1:
                        cond = t
                    else:
                        cond = vtypes.Ands(t, cond)

                statement = [vtypes.If(cond)(*statement)]

            self.delayed_body[delay][index].extend(statement)
            self._add_dst_var(statement)

            return self

        if cond is not None:
            statement = [vtypes.If(cond)(*statement)]
            self.last_if_statement = statement[0]

        self.body[index].extend(statement)
        self._add_dst_var(statement)

        return self

    #-------------------------------------------------------------------------
    def _add_dst_var(self, statement):
        for s in statement:
            values = self.dst_visitor.visit(s)
            for v in values:
                k = str(v)
                if k not in self.dst_var:
                    self.dst_var[k] = v

    #-------------------------------------------------------------------------
    def _add_delayed_cond(self, statement, index, delay):
        name_prefix = '_'.join(
            ['', self.name, 'cond', str(index), str(self.tmp_count)])
        self.tmp_count += 1
        prev = statement
        for i in range(delay):
            tmp_name = '_'.join([name_prefix, str(i + 1)])
            tmp = self.m.Reg(tmp_name, initval=0)
            self._add_statement([tmp(prev)], delay=i, no_delay_cond=True)
            prev = tmp
        return prev

    #-------------------------------------------------------------------------
    def _add_delayed_subst(self, subst, index, delay):
        if not isinstance(subst, vtypes.Subst):
            return subst
        left = subst.left
        right = subst.right
        if isinstance(right, (bool, int, float, str,
                              vtypes._Constant, vtypes._ParameterVariable)):
            return subst
        width = left.bit_length()
        prev = right

        name_prefix = ('_'.join(['', left.name, str(index), str(self.tmp_count)])
                       if isinstance(left, vtypes._Variable) else
                       '_'.join(['', self.name, 'sbst', str(index), str(self.tmp_count)]))
        self.tmp_count += 1

        for i in range(delay):
            tmp_name = '_'.join([name_prefix, str(i + 1)])
            tmp = self.m.Reg(tmp_name, width, initval=0)
            self._add_statement([tmp(prev)], delay=i, no_delay_cond=True)
            prev = tmp
        return left(prev)

    #-------------------------------------------------------------------------
    def _clear_next_kwargs(self):
        self.next_kwargs = {}

    def _clear_last_if_statement(self):
        self.last_if_statement = None

    def _clear_last_cond(self):
        self.last_cond = []

    def _clear_elif_cond(self):
        self.elif_cond = None

    def _make_cond(self, condlist):
        ret = None
        for cond in condlist:
            if ret is None:
                ret = cond
            else:
                ret = vtypes.Ands(ret, cond)
        return ret

    #-------------------------------------------------------------------------
    def _set_index(self, index=None):
        if index is None:
            self.state_count += 1
            return self.state_count
        self.state_count = index
        return self.state_count

    def _get_mark(self, index=None):
        if index is None:
            index = self.state_count
        if index not in self.mark:
            raise KeyError("No such index in FSM marks: %s" % index)
        return self.mark[index]

    def _set_mark(self, index=None, name=None):
        if index is None:
            index = self.state_count
        if name is None:
            name = self.name + '_' + str(index)
        self.mark[index] = self.m.Localparam(name, index)

    def _get_mark_index(self, s):
        for index, m in self.mark.items():
            if m.name == s.name:
                return index
        raise KeyError("No such mark in FSM marks: %s" % s.name)

    #-------------------------------------------------------------------------
    def _add_mark(self, index):
        index = self._to_index(index)
        if index not in self.mark:
            self._set_mark(index)
        mark = self._get_mark(index)
        return mark

    def _to_index(self, index):
        if not isinstance(index, int):
            index = self._get_mark_index(index)
        return index

    #-------------------------------------------------------------------------
    def _add_delayed_state(self, value):
        if not isinstance(value, int):
            raise TypeError("Delay amount must be int, not '%s'" %
                            str(type(value)))

        if value < 0:
            raise ValueError("Delay amount must be positive number")

        if value == 0:
            return self.state

        if value <= self.delay_amount:
            return self._get_delayed_state(value)

        for i in range(self.delay_amount + 1, value + 1):
            d = self.m.Reg(''.join(['_d', str(i), '_', self.name]), self.width,
                           initval=self._get_mark(0))
            self.delayed_state[i] = d

        self.delay_amount = value
        return d

    def _get_delayed_state(self, value):
        if value == 0:
            return self.state
        if value not in self.delayed_state:
            raise IndexError('No such index %d in delayed state' % value)
        return self.delayed_state[value]

    def _get_delayed_substs(self):
        ret = []
        prev = self.state
        for d in range(1, self.delay_amount + 1):
            ret.append(vtypes.Subst(self.delayed_state[d], prev))
            prev = self.delayed_state[d]
        return ret

    def _init_delayed_state(self):
        ret = []
        for d in range(1, self.delay_amount + 1):
            ret.append(vtypes.Subst(self.delayed_state[d], self._get_mark(0)))
        return ret

    def _to_state_assign(self, dst, cond=None, else_dst=None):
        dst_mark = self._get_mark(dst)
        value = self.state(dst_mark)
        if cond is not None:
            value = vtypes.If(cond)(value)
            if else_dst is not None:
                else_dst_mark = self._get_mark(else_dst)
                value = value.Else(self.state(else_dst_mark))
        return value

    #-------------------------------------------------------------------------
    def _cond_case(self, index):
        if index not in self.mark:
            self._set_mark(index)
        return self._get_mark(index)

    def _cond_if(self, index):
        if index not in self.mark:
            self._set_mark(index)
        return (self.state == self._get_mark(index))

    def _delayed_cond_if(self, index, delay):
        if index not in self.mark:
            self._set_mark(index)
        if delay > 0 and delay not in self.delayed_state:
            self._add_delayed_state(delay)
        return (self._get_delayed_state(delay) == self._get_mark(index))

    def _get_when_statement(self, index):
        body = []
        body.extend(self.body[index])
        for dst, cond, else_dst in self.jump[index]:
            self._add_mark(dst)
            if else_dst is not None:
                self._add_mark(else_dst)
            body.append(self._to_state_assign(dst, cond, else_dst))
        return vtypes.When(self._cond_case(index))(*body)

    def _get_delayed_when_statement(self, index, delay):
        return vtypes.When(self._cond_case(index))(*self.delayed_body[delay][index])

    def _get_if_statement(self, index):
        body = []
        body.extend(self.body[index])
        for dst, cond, else_dst in self.jump[index]:
            self._add_mark(dst)
            if else_dst is not None:
                self._add_mark(else_dst)
            body.append(self._to_state_assign(dst, cond, else_dst))
        return vtypes.If(self._cond_if(index))(*body)

    def _get_delayed_if_statement(self, index, delay):
        return vtypes.If(self._delayed_cond_if(index, delay))(*self.delayed_body[delay][index])

    #-------------------------------------------------------------------------
    def __call__(self, *statement, **kwargs):
        return self.add(*statement, **kwargs)

    def __getitem__(self, index):
        return self.body[index]

    def __len__(self):
        return self.state_count + 1



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import simulation_pycoram_userlogic

expected_verilog = """
module test #
(
  parameter DATA_WIDTH = 32,
  parameter ADDR_LEN = 10
);

  reg CLK;
  reg RST;
  wire [ADDR_LEN-1:0] mem_addr;
  wire [DATA_WIDTH-1:0] mem_wdata;
  reg [DATA_WIDTH-1:0] mem_rdata;
  wire mem_wvalid;
  wire mem_rvalid;
  wire [DATA_WIDTH-1:0] ch_wdata;
  wire ch_enq;
  reg ch_almfull;
  reg [DATA_WIDTH-1:0] ch_rdata;
  wire ch_deq;
  reg ch_empty;

  userlogic
  #(
    .DATA_WIDTH(DATA_WIDTH),
    .ADDR_LEN(ADDR_LEN)
  )
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .mem_addr(mem_addr),
    .mem_wdata(mem_wdata),
    .mem_rdata(mem_rdata),
    .mem_wvalid(mem_wvalid),
    .mem_rvalid(mem_rvalid),
    .ch_wdata(ch_wdata),
    .ch_enq(ch_enq),
    .ch_almfull(ch_almfull),
    .ch_rdata(ch_rdata),
    .ch_deq(ch_deq),
    .ch_empty(ch_empty)
  );

  reg [32-1:0] test_fsm;
  localparam test_fsm_init = 0;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut, test_fsm);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    mem_rdata = 0;
    ch_almfull = 0;
    ch_rdata = 0;
    ch_empty = 1;
    test_fsm = test_fsm_init;
    #100;
    RST = 1;
    #100;
    RST = 0;
  end

  reg [32-1:0] _d1_test_fsm;
  reg _test_fsm_cond_12_0_1;
  localparam test_fsm_1 = 1;
  localparam test_fsm_2 = 2;
  localparam test_fsm_3 = 3;
  localparam test_fsm_4 = 4;
  localparam test_fsm_5 = 5;
  localparam test_fsm_6 = 6;
  localparam test_fsm_7 = 7;
  localparam test_fsm_8 = 8;
  localparam test_fsm_9 = 9;
  localparam test_fsm_10 = 10;
  localparam test_fsm_11 = 11;
  localparam test_fsm_12 = 12;
  localparam test_fsm_13 = 13;

  always @(posedge CLK) begin
    if(RST) begin
      test_fsm <= test_fsm_init;
      _d1_test_fsm <= test_fsm_init;
      ch_empty <= 1;
      ch_rdata <= 0;
      mem_rdata <= 0;
      _test_fsm_cond_12_0_1 <= 0;
    end else begin
      _d1_test_fsm <= test_fsm;
      case(_d1_test_fsm)
        test_fsm_12: begin
          if(_test_fsm_cond_12_0_1) begin
            mem_rdata <= mem_rdata + 1;
          end 
        end
      endcase
      case(test_fsm)
        test_fsm_init: begin
          test_fsm <= test_fsm_1;
        end
        test_fsm_1: begin
          test_fsm <= test_fsm_2;
        end
        test_fsm_2: begin
          test_fsm <= test_fsm_3;
        end
        test_fsm_3: begin
          test_fsm <= test_fsm_4;
        end
        test_fsm_4: begin
          test_fsm <= test_fsm_5;
        end
        test_fsm_5: begin
          test_fsm <= test_fsm_6;
        end
        test_fsm_6: begin
          test_fsm <= test_fsm_7;
        end
        test_fsm_7: begin
          test_fsm <= test_fsm_8;
        end
        test_fsm_8: begin
          test_fsm <= test_fsm_9;
        end
        test_fsm_9: begin
          test_fsm <= test_fsm_10;
        end
        test_fsm_10: begin
          ch_empty <= 0;
          ch_rdata <= 10;
          test_fsm <= test_fsm_11;
        end
        test_fsm_11: begin
          if(ch_deq) begin
            ch_empty <= 1;
          end 
          mem_rdata <= 0;
          if(ch_deq) begin
            test_fsm <= test_fsm_12;
          end 
        end
        test_fsm_12: begin
          _test_fsm_cond_12_0_1 <= mem_rvalid;
          if(ch_enq) begin
            test_fsm <= test_fsm_13;
          end 
        end
        test_fsm_13: begin
          $finish;
        end
      endcase
    end
  end


endmodule



module userlogic #
(
  parameter DATA_WIDTH = 32,
  parameter ADDR_LEN = 10
)
(
  input CLK,
  input RST,
  output reg [ADDR_LEN-1:0] mem_addr,
  output reg [DATA_WIDTH-1:0] mem_wdata,
  input [DATA_WIDTH-1:0] mem_rdata,
  output reg mem_wvalid,
  output reg mem_rvalid,
  output reg [DATA_WIDTH-1:0] ch_wdata,
  output reg ch_enq,
  input ch_almfull,
  input [DATA_WIDTH-1:0] ch_rdata,
  output reg ch_deq,
  input ch_empty
);

  reg [32-1:0] fsm;
  localparam fsm_init = 0;
  reg [32-1:0] read_count;
  reg [32-1:0] sum;
  reg [32-1:0] size;
  reg [32-1:0] _d1_fsm;
  reg _fsm_cond_2_0_1;
  reg [32-1:0] _d2_fsm;
  reg _fsm_cond_2_1_1;
  reg _fsm_cond_2_1_2;
  reg _fsm_cond_4_2_1;
  reg _fsm_cond_4_3_1;
  reg _fsm_cond_4_3_2;
  reg [32-1:0] _d3_fsm;
  reg _fsm_cond_4_4_1;
  reg _fsm_cond_4_4_2;
  reg _fsm_cond_4_4_3;
  reg _fsm_cond_6_5_1;
  reg _fsm_cond_6_5_2;
  reg _fsm_cond_6_6_1;
  localparam fsm_1 = 1;
  localparam fsm_2 = 2;
  localparam fsm_3 = 3;
  localparam fsm_4 = 4;
  localparam fsm_5 = 5;
  localparam fsm_6 = 6;

  always @(posedge CLK) begin
    if(RST) begin
      mem_addr <= 0;
      mem_wdata <= 0;
      mem_wvalid <= 0;
      mem_rvalid <= 0;
      ch_wdata <= 0;
      ch_enq <= 0;
      ch_deq <= 0;
      fsm <= fsm_init;
      read_count <= 0;
      sum <= 0;
      size <= 0;
      _d1_fsm <= fsm_init;
      _fsm_cond_2_0_1 <= 0;
      _d2_fsm <= fsm_init;
      _fsm_cond_2_1_1 <= 0;
      _fsm_cond_2_1_2 <= 0;
      _fsm_cond_4_2_1 <= 0;
      _fsm_cond_4_3_1 <= 0;
      _fsm_cond_4_3_2 <= 0;
      _d3_fsm <= fsm_init;
      _fsm_cond_4_4_1 <= 0;
      _fsm_cond_4_4_2 <= 0;
      _fsm_cond_4_4_3 <= 0;
      _fsm_cond_6_5_1 <= 0;
      _fsm_cond_6_5_2 <= 0;
      _fsm_cond_6_6_1 <= 0;
    end else begin
      _d1_fsm <= fsm;
      _d2_fsm <= _d1_fsm;
      _d3_fsm <= _d2_fsm;
      case(_d3_fsm)
        fsm_4: begin
          if(_fsm_cond_4_4_3) begin
            sum <= sum + mem_rdata;
          end 
        end
      endcase
      case(_d2_fsm)
        fsm_2: begin
          if(_fsm_cond_2_1_2) begin
            size <= ch_rdata;
          end 
        end
        fsm_4: begin
          if(_fsm_cond_4_3_2) begin
            sum <= sum + mem_rdata;
          end 
          _fsm_cond_4_4_3 <= _fsm_cond_4_4_2;
        end
        fsm_6: begin
          if(_fsm_cond_6_5_2) begin
            $display("sum=%d", sum);
          end 
        end
      endcase
      case(_d1_fsm)
        fsm_2: begin
          if(_fsm_cond_2_0_1) begin
            ch_deq <= 0;
          end 
          _fsm_cond_2_1_2 <= _fsm_cond_2_1_1;
        end
        fsm_4: begin
          if(_fsm_cond_4_2_1) begin
            mem_rvalid <= 0;
          end 
          _fsm_cond_4_3_2 <= _fsm_cond_4_3_1;
          _fsm_cond_4_4_2 <= _fsm_cond_4_4_1;
        end
        fsm_6: begin
          _fsm_cond_6_5_2 <= _fsm_cond_6_5_1;
          if(_fsm_cond_6_6_1) begin
            ch_enq <= 0;
          end 
        end
      endcase
      case(fsm)
        fsm_init: begin
          fsm <= fsm_1;
        end
        fsm_1: begin
          fsm <= fsm_2;
        end
        fsm_2: begin
          if(!ch_empty) begin
            ch_deq <= 1;
          end 
          _fsm_cond_2_0_1 <= 1;
          _fsm_cond_2_1_1 <= 1;
          if(!ch_empty) begin
            fsm <= fsm_3;
          end 
        end
        fsm_3: begin
          mem_addr <= -1;
          fsm <= fsm_4;
        end
        fsm_4: begin
          mem_rvalid <= 1;
          mem_addr <= mem_addr + 1;
          read_count <= read_count + 1;
          _fsm_cond_4_2_1 <= 1;
          _fsm_cond_4_3_1 <= 1;
          _fsm_cond_4_4_1 <= 1;
          if(read_count == size - 1) begin
            fsm <= fsm_5;
          end 
        end
        fsm_5: begin
          fsm <= fsm_6;
        end
        fsm_6: begin
          _fsm_cond_6_5_1 <= 1;
          if(!ch_almfull) begin
            ch_enq <= 1;
          end 
          _fsm_cond_6_6_1 <= 1;
          ch_wdata <= sum;
          fsm <= fsm_2;
        end
      endcase
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = simulation_pycoram_userlogic.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *
import veriloggen.dataflow as dataflow

def mkMain(n=128, datawidth=32, numports=2):
    m = Module('main')

    clk = m.Input('CLK')
    rst = m.Input('RST')

    a = dataflow.Parameter('aparam', 10) # parameter
    b = dataflow.ParameterVariable('bparam') # input / wire
    
    xparam = m.Parameter('xparam', 100)
    x = dataflow.ParameterVariable(xparam)

    yparam = m.Reg('yparam', 32, initval=0)
    y = dataflow.ParameterVariable(yparam)

    zparam = 30
    z = dataflow.ParameterVariable(zparam)

    v = dataflow.Variable('vdata')
    w = dataflow.Variable('wdata')

    c = v + a + b + x + y + 1 + z + w 
    c.output('cdata', valid='cvalid')

    # synthesize dataflow
    df = dataflow.Dataflow(c)
    df.implement(m, clk, rst, aswire=True)
    #df.draw_graph()

    fsm = FSM(m, 'fsm', clk, rst)
    count = m.TmpReg(32, initval=0)

    b.raw_data.assign(20)
    v.raw_data.assign(1000)
    w.raw_data.assign(2000)
    
    fsm(
        yparam(200)
    )
    fsm.goto_next()
    
    fsm.If(c.valid)(
        count.inc(),
        Systask('display', "c=%d", c.data)
    )

    fsm.If(count == 4).goto_next()
    
    fsm.make_always()
    
    return m
    
def mkTest():
    m = Module('test')
    
    # target instance
    main = mkMain()
    
    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)
    
    clk = ports['CLK']
    rst = ports['RST']
    
    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))
    
    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)
    
    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m
    
if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    #sim.view_waveform()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
import copy
import collections
import functools

import veriloggen.core.vtypes as vtypes
from veriloggen.core.module import Module
from veriloggen.seq.seq import Seq

from . import visitor
from . import dtypes
from . import mul
from . import scheduler
from . import allocator
from . import graph


# ID counter for 'Dataflow'
_dataflow_counter = 0


def reset():
    global _dataflow_counter
    _dataflow_counter = 0
    dtypes._object_counter = 0
    mul.reset()


def DataflowManager(module, clock, reset, no_hook=False):
    return Dataflow(module=module, clock=clock, reset=reset,
                    no_hook=no_hook)


class Dataflow(object):

    def __init__(self, *nodes, **opts):
        # ID for manager reuse and merge
        global _dataflow_counter
        self.object_id = _dataflow_counter
        _dataflow_counter += 1

        self.nodes = set(nodes)
        self.max_stage = None
        self.last_input = None
        self.last_output = None

        self.module = opts['module'] if 'module' in opts else None
        self.clock = opts['clock'] if 'clock' in opts else None
        self.reset = opts['reset'] if 'reset' in opts else None
        self.seq = None

        if (self.module is not None and
                self.clock is not None and self.reset is not None):

            no_hook = opts['no_hook'] if 'no_hook' in opts else False
            if not no_hook:
                self.module.add_hook(self.implement)

            seq_name = (('_dataflow_seq_%d' % self.object_id) if 'seq_name' not in opts else
                        opts['seq_name'])
            self.seq = Seq(self.module, seq_name, self.clock, self.reset)

    #-------------------------------------------------------------------------
    def add(self, *nodes):
        self.nodes.update(set(nodes))

    #-------------------------------------------------------------------------
    def to_module(self, name, clock='CLK', reset='RST'):
        """ generate a Module definion """

        m = Module(name)
        clk = m.Input(clock)
        rst = m.Input(reset)

        m = self.implement(m, clk, rst, aswire=False)

        return m

    #-------------------------------------------------------------------------
    def implement(self, m=None, clock=None, reset=None, seq_name=None, aswire=True):
        """ implemente actual registers and operations in Verilog """

        if m is None:
            m = self.module

        if clock is None:
            clock = self.clock

        if reset is None:
            reset = self.reset

        if self.seq is None:
            if seq_name is None:
                seq_name = '_dataflow_seq_%d' % self.object_id
            seq = Seq(m, seq_name, clock, reset)
        else:
            seq = self.seq

        # for mult and div
        m._clock = clock
        m._reset = reset

        dataflow_nodes = self.nodes

        input_visitor = visitor.InputVisitor()
        input_vars = set()
        for node in sorted(dataflow_nodes, key=lambda x: x.object_id):
            input_vars.update(input_visitor.visit(node))

        output_visitor = visitor.OutputVisitor()
        output_vars = set()
        for node in sorted(dataflow_nodes, key=lambda x: x.object_id):
            output_vars.update(output_visitor.visit(node))

        # add input ports
        for input_var in sorted(input_vars, key=lambda x: x.object_id):
            input_var._implement_input(m, seq, aswire)

        # schedule
        sched = scheduler.ASAPScheduler()
        sched.schedule(output_vars)

        # balance output stage depth
        max_stage = None
        for output_var in sorted(output_vars, key=lambda x: x.object_id):
            max_stage = dtypes._max(max_stage, output_var.end_stage)
        self.max_stage = max_stage

        output_vars = sched.balance_output(output_vars, max_stage)

        # get all vars
        all_visitor = visitor.AllVisitor()
        all_vars = set()
        for output_var in sorted(output_vars, key=lambda x: x.object_id):
            all_vars.update(all_visitor.visit(output_var))

        # allocate (implement signals)
        alloc = allocator.Allocator()
        alloc.allocate(m, seq, all_vars)

        # set default module information
        for var in sorted(all_vars, key=lambda x: x.object_id):
            var._set_module(m)
            var._set_df(self)

            if var.seq is not None:
                seq.update(var.seq)

            var._set_seq(seq)

        # add output ports
        for output_var in sorted(output_vars, key=lambda x: x.object_id):
            output_var._implement_output(m, seq, aswire)

        # save schedule result
        self.last_input = input_vars
        self.last_output = output_vars

        return m

    #-------------------------------------------------------------------------
    def draw_graph(self, filename='out.png', prog='dot', rankdir='LR', approx=False):
        if self.last_output is None:
            self.to_module()

        graph.draw_graph(self.last_output, filename=filename, prog=prog,
                         rankdir=rankdir, approx=approx)

    def enable_draw_graph(self, filename='out.png', prog='dot', rankdir='LR', approx=False):
        self.module.add_hook(self.draw_graph,
                             kwargs={'filename': filename, 'prog': prog,
                                     'rankdir': rankdir, 'approx': approx})

    #-------------------------------------------------------------------------
    def get_input(self):
        if self.last_input is None:
            return collections.OrderedDict()

        ret = collections.OrderedDict()
        for input_var in sorted(self.last_input, key=lambda x: x.object_id):
            key = str(input_var.input_data)
            value = input_var
            ret[key] = value

        return ret

    def get_output(self):
        if self.last_output is None:
            return collections.OrderedDict()

        ret = collections.OrderedDict()
        for output_var in sorted(self.last_output, key=lambda x: x.object_id):
            key = str(output_var.output_data)
            value = output_var
            ret[key] = value

        return ret

    #-------------------------------------------------------------------------
    def pipeline_depth(self):
        return self.max_stage

    #-------------------------------------------------------------------------
    def __getattr__(self, attr):
        try:
            return object.__getattr__(self, attr)

        except AttributeError as e:
            if attr.startswith('__') or attr not in dir(dtypes):
                raise e

            func = getattr(dtypes, attr)

            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                v = func(*args, **kwargs)
                if isinstance(v, (tuple, list)):
                    for item in v:
                        self._set_info(item)
                else:
                    self._set_info(v)
                return v

            return wrapper

    def _set_info(self, v):
        if isinstance(v, dtypes._Numeric):
            v._set_module(self.module)
            v._set_df(self)
            v._set_seq(self.seq)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))))

from veriloggen import *
import veriloggen.types.mul as mul


def mkMain(n=128, datawidth=32, numports=2):
    m = Module('main')

    clk = m.Input('CLK')
    rst = m.Input('RST')

    a = m.Reg('a', 16, initval=0)
    b = m.Reg('b', 16, initval=0)
    c = m.Wire('c', a.bit_length() + b.bit_length())

    mymul = mul.Multiplier(m, 'mymul', clk, rst, a, b)

    seq = Seq(m, 'seq', clk, rst)

    seq(
        a.add(1),
        b.add(1)
    )

    seq(
        Display("mymul.value = %d", mymul.value)
    )

    return m


def mkTest():
    m = Module('test')

    # target instance
    main = mkMain()

    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(1000),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    # sim.view_waveform()



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *

def mkLed():
    m = Module('blinkled')
    width = m.Parameter('WIDTH', 8)
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', width, initval=0)
    count = m.Reg('count', 32, initval=0)

    fsm = FSM(m, 'fsm', clk, rst)
    # get the initial index (= 0)
    init = fsm.current

    fsm( count(count + 1) )
    fsm.goto_next()
    fsm( count(count + 2) )
    fsm.goto_next()
    
    # get the current index (= 2)
    here = fsm.current
    fsm( count(count + 3) )
    fsm.goto_next()

    # jump by using label "init"
    fsm.goto( dst=init, cond=(count < 1024), else_dst=fsm.next ).inc()
    
    fsm( led(led + 1) )
    # jump by using label "here"
    fsm.goto(here)

    fsm.make_always()

    return m

if __name__ == '__main__':
    led = mkLed()
    verilog = led.to_verilog()
    print(verilog)
